#open "obj";;
#open "lexing";;


(**) #open "parser";;

(* For nested comments *)

let comment_depth = ref 0;;

(* The table of keywords *)

let keyword_table = (hashtbl__new 149 : (string, token) hashtbl__t)
;;

do_list (fun (str,tok) -> hashtbl__add keyword_table str tok) [
  "and", AND;
  "as", AS;
  "begin", BEGIN;
  "do", DO;
  "done", DONE;
  "downto", DOWNTO;
  "else", ELSE;
  "end", END;
  "exception", EXCEPTION;
  "for", FOR;
  "fun", FUN;
  "function", FUNCTION;
  "if", IF;
  "in", IN;
  "let", LET;
  "match", MATCH;
  "mutable", MUTABLE;
  "not", NOT;
  "of", OF;
  "or", OR;
  "prefix", PREF;
  "rec", REC;
  "then", THEN;
  "to", TO;
  "try", TRY;
  "type", TYPE;
  "value", VALUE;
  "when", WHEN;
  "where", WHERE;
  "while", WHILE;
  "with", WITH;

  "quo", INFIX3("quo");
  "mod", INFIX3("mod");
  "land", INFIX3("land");
  "lor", INFIX2("lor");
  "lxor", INFIX2("lxor");
  "lsl", INFIX4("lsl");
  "lsr", INFIX4("lsr");
  "asr", INFIX4("asr")
];;

let add_infix s =
  hashtbl__add keyword_table s (INFIX s)
;;

let remove_infix s =
  hashtbl__remove keyword_table s
;;


(* To buffer string literals *)

let initial_string_buffer = create_string 256;;
let string_buff = ref initial_string_buffer;;
let string_index = ref 0;;

let reset_string_buffer () =
  string_buff := initial_string_buffer;
  string_index := 0;
  ()
;;

let store_string_char c =
  if !string_index >= string_length (!string_buff) then begin
    let new_buff = create_string (string_length (!string_buff) * 2) in
      blit_string (!string_buff) 0 new_buff 0 (string_length (!string_buff));
      string_buff := new_buff
  end;
  set_nth_char (!string_buff) (!string_index) c;
  incr string_index
;;

let get_stored_string () =
  let s = sub_string (!string_buff) 0 (!string_index) in
    string_buff := initial_string_buffer;
    s
;;

(* To translate escape sequences *)

let char_for_backslash = function

    `n` -> `\010`
  | `r` -> `\013`

  | `b` -> `\008`
  | `t` -> `\009`
  | c   -> c
;;

let char_for_decimal_code lexbuf i =
  let c = 
    100 * (int_of_char(get_lexeme_char lexbuf i) - 48) +
     10 * (int_of_char(get_lexeme_char lexbuf (i+1)) - 48) +
          (int_of_char(get_lexeme_char lexbuf (i+2)) - 48) in
  char_of_int(c land 0xFF)
;;


let rec action_69 lexbuf = (
 raise (Lexical_error(Illegal_character,
                            get_lexeme_start lexbuf, get_lexeme_end lexbuf)) )
and action_68 lexbuf = (
 EOF )
and action_67 lexbuf = (
 INFIX3(get_lexeme lexbuf) )
and action_66 lexbuf = (
 INFIX4(get_lexeme lexbuf) )
and action_65 lexbuf = (
 INFIX2(get_lexeme lexbuf) )
and action_64 lexbuf = (
 INFIX1(get_lexeme lexbuf) )
and action_63 lexbuf = (
 INFIX0(get_lexeme lexbuf) )
and action_62 lexbuf = (
 PREFIX(get_lexeme lexbuf) )
and action_61 lexbuf = (
 SUBTRACTIVE "-." )
and action_60 lexbuf = (
 SUBTRACTIVE "-" )
and action_59 lexbuf = (
 INFIX0 "!=" )
and action_58 lexbuf = (
 BARBAR )
and action_57 lexbuf = (
 AMPERAMPER )
and action_56 lexbuf = (
 RBRACE )
and action_55 lexbuf = (
 BARRBRACKET )
and action_54 lexbuf = (
 BAR )
and action_53 lexbuf = (
 LBRACE )
and action_52 lexbuf = (
 UNDERUNDER )
and action_51 lexbuf = (
 UNDERSCORE )
and action_50 lexbuf = (
 RBRACKET )
and action_49 lexbuf = (
 LBRACKETLESS )
and action_48 lexbuf = (
 LBRACKETBAR )
and action_47 lexbuf = (
 LBRACKET )
and action_46 lexbuf = (
 GREATERRBRACKET )
and action_45 lexbuf = (
 EQUALEQUAL )
and action_44 lexbuf = (
 EQUAL )
and action_43 lexbuf = (
 LESSMINUS )
and action_42 lexbuf = (
 SEMISEMI )
and action_41 lexbuf = (
 SEMI )
and action_40 lexbuf = (
 COLONEQUAL )
and action_39 lexbuf = (
 COLONCOLON )
and action_38 lexbuf = (
 COLON )
and action_37 lexbuf = (
 DOTLBRACKET )
and action_36 lexbuf = (
 DOTLPAREN )
and action_35 lexbuf = (
 DOTDOT )
and action_34 lexbuf = (
 DOT )
and action_33 lexbuf = (
 MINUSGREATER )
and action_32 lexbuf = (
 COMMA )
and action_31 lexbuf = (
 STAR )
and action_30 lexbuf = (
 RPAREN )
and action_29 lexbuf = (
 LPAREN )
and action_28 lexbuf = (
 QUOTE )
and action_27 lexbuf = (
 AMPERSAND )
and action_26 lexbuf = (
 SHARP )
and action_25 lexbuf = (
 let comment_start = lexbuf.lex_start_pos + lexbuf.lex_abs_pos in
        comment_depth := 1;
        begin try
          comment lexbuf
        with Lexical_error(Unterminated_comment, _, comment_end) ->
          raise(Lexical_error(Unterminated_comment,
                              comment_start, comment_end))
        end;
        main lexbuf )
and action_24 lexbuf = (
 let char_start = lexbuf.lex_start_pos + lexbuf.lex_abs_pos in
        let c = char lexbuf in
        lexbuf.lex_start_pos <- char_start - lexbuf.lex_abs_pos;
        CHAR c )
and action_23 lexbuf = (
 reset_string_buffer();
        let string_start = lexbuf.lex_start_pos + lexbuf.lex_abs_pos in
        begin try
          string lexbuf
        with Lexical_error(Unterminated_string, _, string_end) ->
          raise(Lexical_error(Unterminated_string, string_start, string_end))
        end;
        lexbuf.lex_start_pos <- string_start - lexbuf.lex_abs_pos;
        STRING (get_stored_string()) )
and action_22 lexbuf = (
 FLOAT (float_of_string(get_lexeme lexbuf)) )
and action_21 lexbuf = (
 INT (int_of_string(get_lexeme lexbuf)) )
and action_20 lexbuf = (
 let s = get_lexeme lexbuf in
          try
            hashtbl__find keyword_table s
          with Not_found ->
            IDENT s )
and action_19 lexbuf = (
 main lexbuf )
and action_18 lexbuf = (
 comment lexbuf )
and action_17 lexbuf = (
 raise(Lexical_error
                (Unterminated_comment, 0, get_lexeme_start lexbuf)) )
and action_16 lexbuf = (
 comment lexbuf )
and action_15 lexbuf = (
 comment lexbuf )
and action_14 lexbuf = (
 comment lexbuf )
and action_13 lexbuf = (
 comment lexbuf )
and action_12 lexbuf = (
 reset_string_buffer();
        let string_start = lexbuf.lex_start_pos + lexbuf.lex_abs_pos in
        begin try
          string lexbuf
        with Lexical_error(Unterminated_string, _, string_end) ->
          raise(Lexical_error(Unterminated_string, string_start, string_end))
        end;
        comment lexbuf )
and action_11 lexbuf = (
 comment_depth := pred !comment_depth;
        if !comment_depth > 0 then comment lexbuf )
and action_10 lexbuf = (
 comment_depth := succ !comment_depth; comment lexbuf )
and action_9 lexbuf = (
 raise (Lexical_error(Bad_char_constant,
                            get_lexeme_start lexbuf - 1,
                            get_lexeme_end lexbuf)) )
and action_8 lexbuf = (
 char_for_decimal_code lexbuf 1 )
and action_7 lexbuf = (
 char_for_backslash (get_lexeme_char lexbuf 1) )
and action_6 lexbuf = (
 get_lexeme_char lexbuf 0 )
and action_5 lexbuf = (
 store_string_char(get_lexeme_char lexbuf 0);
        string lexbuf )
and action_4 lexbuf = (
 raise (Lexical_error
                (Unterminated_string, 0, get_lexeme_start lexbuf)) )
and action_3 lexbuf = (
 store_string_char(char_for_decimal_code lexbuf 1);
         string lexbuf )
and action_2 lexbuf = (
 store_string_char(char_for_backslash(get_lexeme_char lexbuf 1));
        string lexbuf )
and action_1 lexbuf = (
 string lexbuf )
and action_0 lexbuf = (
 () )
and state_0 lexbuf =
  match get_next_char lexbuf with
    `\\` -> state_111 lexbuf
 |  `"` -> action_0 lexbuf
 |  `\000` -> action_4 lexbuf
 |  _ -> action_5 lexbuf
and state_1 lexbuf =
  match get_next_char lexbuf with
    `\``|`\000` -> action_9 lexbuf
 |  `\\` -> state_97 lexbuf
 |  _ -> state_96 lexbuf
and state_2 lexbuf =
  match get_next_char lexbuf with
    `\`` -> state_82 lexbuf
 |  `*` -> state_81 lexbuf
 |  `(` -> state_80 lexbuf
 |  `"` -> action_12 lexbuf
 |  `\000` -> action_17 lexbuf
 |  _ -> action_18 lexbuf
and state_3 lexbuf =
  match get_next_char lexbuf with
    ``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|`\160`|`\159`|`\158`|`\157`|`\156`|`\155`|`\154`|`\153`|`\152`|`\151`|`\150`|`\149`|`\148`|`\147`|`\146`|`\145`|`\144`|`\143`|`\142`|`\141`|`\140`|`\139`|`\138`|`\137`|`\136`|`\135`|`\134`|`\133`|`\132`|`\131`|`\130`|`\129`|`\128`|`\127`|`\\`|`\031`|`\030`|`\029`|`\028`|`\027`|`\026`|`\025`|`\024`|`\023`|`\022`|`\021`|`\020`|`\019`|`\018`|`\017`|`\016`|`\015`|`\014`|`\011`|`\008`|`\007`|`\006`|`\005`|`\004`|`\003`|`\002`|`\001` -> action_69 lexbuf
 |  `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1` -> state_22 lexbuf
 |  ` `|`\013`|`\012`|`\010`|`\t` -> state_6 lexbuf
 |  `^`|`@` -> state_29 lexbuf
 |  `/`|`%` -> state_11 lexbuf
 |  `~`|`$` -> state_10 lexbuf
 |  `}` -> action_56 lexbuf
 |  `|` -> state_36 lexbuf
 |  `{` -> action_53 lexbuf
 |  `\`` -> action_24 lexbuf
 |  `_` -> state_33 lexbuf
 |  `]` -> action_50 lexbuf
 |  `[` -> state_31 lexbuf
 |  `?` -> state_28 lexbuf
 |  `>` -> state_27 lexbuf
 |  `=` -> state_26 lexbuf
 |  `<` -> state_25 lexbuf
 |  `;` -> state_24 lexbuf
 |  `:` -> state_23 lexbuf
 |  `0` -> state_21 lexbuf
 |  `.` -> state_20 lexbuf
 |  `-` -> state_19 lexbuf
 |  `,` -> action_32 lexbuf
 |  `+` -> state_17 lexbuf
 |  `*` -> state_16 lexbuf
 |  `)` -> action_30 lexbuf
 |  `(` -> state_14 lexbuf
 |  `'` -> action_28 lexbuf
 |  `&` -> state_12 lexbuf
 |  `#` -> action_26 lexbuf
 |  `"` -> action_23 lexbuf
 |  `!` -> state_7 lexbuf
 |  `\000` -> action_68 lexbuf
 |  _ -> state_30 lexbuf
and state_6 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_19;
  match get_next_char lexbuf with
    ` `|`\013`|`\012`|`\010`|`\t` -> state_76 lexbuf
 |  _ -> backtrack lexbuf
and state_7 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_62;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_47 lexbuf
 |  `=` -> state_75 lexbuf
 |  _ -> backtrack lexbuf
and state_10 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_63;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  _ -> backtrack lexbuf
and state_11 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_67;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_71 lexbuf
 |  _ -> backtrack lexbuf
and state_12 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_27;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`%`|`$`|`!` -> state_38 lexbuf
 |  `&` -> state_74 lexbuf
 |  _ -> backtrack lexbuf
and state_14 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_29;
  match get_next_char lexbuf with
    `*` -> action_25 lexbuf
 |  _ -> backtrack lexbuf
and state_16 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_31;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`&`|`%`|`$`|`!` -> state_71 lexbuf
 |  `*` -> state_72 lexbuf
 |  _ -> backtrack lexbuf
and state_17 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_65;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_68 lexbuf
 |  _ -> backtrack lexbuf
and state_19 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_60;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`=`|`<`|`:`|`/`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_68 lexbuf
 |  `>` -> state_70 lexbuf
 |  `.` -> state_69 lexbuf
 |  _ -> backtrack lexbuf
and state_20 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_34;
  match get_next_char lexbuf with
    `[` -> action_37 lexbuf
 |  `.` -> action_35 lexbuf
 |  `(` -> action_36 lexbuf
 |  _ -> backtrack lexbuf
and state_21 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_21;
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_55 lexbuf
 |  `x`|`X` -> state_61 lexbuf
 |  `o`|`O` -> state_60 lexbuf
 |  `e`|`E` -> state_56 lexbuf
 |  `b`|`B` -> state_59 lexbuf
 |  `.` -> state_54 lexbuf
 |  _ -> backtrack lexbuf
and state_22 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_21;
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_55 lexbuf
 |  `e`|`E` -> state_56 lexbuf
 |  `.` -> state_54 lexbuf
 |  _ -> backtrack lexbuf
and state_23 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_38;
  match get_next_char lexbuf with
    `=` -> action_40 lexbuf
 |  `:` -> action_39 lexbuf
 |  _ -> backtrack lexbuf
and state_24 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_41;
  match get_next_char lexbuf with
    `;` -> action_42 lexbuf
 |  _ -> backtrack lexbuf
and state_25 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_63;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  `-` -> state_50 lexbuf
 |  _ -> backtrack lexbuf
and state_26 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_44;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  `=` -> state_49 lexbuf
 |  _ -> backtrack lexbuf
and state_27 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_63;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  `]` -> action_46 lexbuf
 |  _ -> backtrack lexbuf
and state_28 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_62;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_47 lexbuf
 |  _ -> backtrack lexbuf
and state_29 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_64;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_46 lexbuf
 |  _ -> backtrack lexbuf
and state_30 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_20;
  match get_next_char lexbuf with
    ``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|`z`|`y`|`x`|`w`|`v`|`u`|`t`|`s`|`r`|`q`|`p`|`o`|`n`|`m`|`l`|`k`|`j`|`i`|`h`|`g`|`f`|`e`|`d`|`c`|`b`|`a`|`Z`|`Y`|`X`|`W`|`V`|`U`|`T`|`S`|`R`|`Q`|`P`|`O`|`N`|`M`|`L`|`K`|`J`|`I`|`H`|`G`|`F`|`E`|`D`|`C`|`B`|`A`|`9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0`|`'` -> state_44 lexbuf
 |  `_` -> state_45 lexbuf
 |  _ -> backtrack lexbuf
and state_31 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_47;
  match get_next_char lexbuf with
    `|` -> action_48 lexbuf
 |  `<` -> action_49 lexbuf
 |  _ -> backtrack lexbuf
and state_33 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_51;
  match get_next_char lexbuf with
    `_` -> action_52 lexbuf
 |  _ -> backtrack lexbuf
and state_36 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_54;
  match get_next_char lexbuf with
    `~`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  `|` -> state_40 lexbuf
 |  `]` -> action_55 lexbuf
 |  _ -> backtrack lexbuf
and state_38 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_63;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  _ -> backtrack lexbuf
and state_40 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_58;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  _ -> backtrack lexbuf
and state_44 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_20;
  match get_next_char lexbuf with
    ``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|`z`|`y`|`x`|`w`|`v`|`u`|`t`|`s`|`r`|`q`|`p`|`o`|`n`|`m`|`l`|`k`|`j`|`i`|`h`|`g`|`f`|`e`|`d`|`c`|`b`|`a`|`Z`|`Y`|`X`|`W`|`V`|`U`|`T`|`S`|`R`|`Q`|`P`|`O`|`N`|`M`|`L`|`K`|`J`|`I`|`H`|`G`|`F`|`E`|`D`|`C`|`B`|`A`|`9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0`|`'` -> state_44 lexbuf
 |  `_` -> state_45 lexbuf
 |  _ -> backtrack lexbuf
and state_45 lexbuf =
  match get_next_char lexbuf with
    ``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|``|`z`|`y`|`x`|`w`|`v`|`u`|`t`|`s`|`r`|`q`|`p`|`o`|`n`|`m`|`l`|`k`|`j`|`i`|`h`|`g`|`f`|`e`|`d`|`c`|`b`|`a`|`Z`|`Y`|`X`|`W`|`V`|`U`|`T`|`S`|`R`|`Q`|`P`|`O`|`N`|`M`|`L`|`K`|`J`|`I`|`H`|`G`|`F`|`E`|`D`|`C`|`B`|`A`|`9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0`|`'` -> state_44 lexbuf
 |  _ -> backtrack lexbuf
and state_46 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_64;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_46 lexbuf
 |  _ -> backtrack lexbuf
and state_47 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_62;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_47 lexbuf
 |  _ -> backtrack lexbuf
and state_49 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_45;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  _ -> backtrack lexbuf
and state_50 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_43;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  _ -> backtrack lexbuf
and state_54 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_22;
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_54 lexbuf
 |  `e`|`E` -> state_56 lexbuf
 |  _ -> backtrack lexbuf
and state_55 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_21;
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_55 lexbuf
 |  `e`|`E` -> state_56 lexbuf
 |  `.` -> state_54 lexbuf
 |  _ -> backtrack lexbuf
and state_56 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_58 lexbuf
 |  `-`|`+` -> state_57 lexbuf
 |  _ -> backtrack lexbuf
and state_57 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_58 lexbuf
 |  _ -> backtrack lexbuf
and state_58 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_22;
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_58 lexbuf
 |  _ -> backtrack lexbuf
and state_59 lexbuf =
  match get_next_char lexbuf with
    `1`|`0` -> state_64 lexbuf
 |  _ -> backtrack lexbuf
and state_60 lexbuf =
  match get_next_char lexbuf with
    `7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_63 lexbuf
 |  _ -> backtrack lexbuf
and state_61 lexbuf =
  match get_next_char lexbuf with
    `f`|`e`|`d`|`c`|`b`|`a`|`F`|`E`|`D`|`C`|`B`|`A`|`9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_62 lexbuf
 |  _ -> backtrack lexbuf
and state_62 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_21;
  match get_next_char lexbuf with
    `f`|`e`|`d`|`c`|`b`|`a`|`F`|`E`|`D`|`C`|`B`|`A`|`9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_62 lexbuf
 |  _ -> backtrack lexbuf
and state_63 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_21;
  match get_next_char lexbuf with
    `7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_63 lexbuf
 |  _ -> backtrack lexbuf
and state_64 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_21;
  match get_next_char lexbuf with
    `1`|`0` -> state_64 lexbuf
 |  _ -> backtrack lexbuf
and state_68 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_65;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_68 lexbuf
 |  _ -> backtrack lexbuf
and state_69 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_61;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_68 lexbuf
 |  _ -> backtrack lexbuf
and state_70 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_33;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_68 lexbuf
 |  _ -> backtrack lexbuf
and state_71 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_67;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_71 lexbuf
 |  _ -> backtrack lexbuf
and state_72 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_66;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_72 lexbuf
 |  _ -> backtrack lexbuf
and state_74 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_57;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_38 lexbuf
 |  _ -> backtrack lexbuf
and state_75 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_59;
  match get_next_char lexbuf with
    `~`|`|`|`^`|`@`|`?`|`>`|`=`|`<`|`:`|`/`|`.`|`-`|`+`|`*`|`&`|`%`|`$`|`!` -> state_47 lexbuf
 |  _ -> backtrack lexbuf
and state_76 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_19;
  match get_next_char lexbuf with
    ` `|`\013`|`\012`|`\010`|`\t` -> state_76 lexbuf
 |  _ -> backtrack lexbuf
and state_80 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_18;
  match get_next_char lexbuf with
    `*` -> action_10 lexbuf
 |  _ -> backtrack lexbuf
and state_81 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_18;
  match get_next_char lexbuf with
    `)` -> action_11 lexbuf
 |  _ -> backtrack lexbuf
and state_82 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_18;
  match get_next_char lexbuf with
    `\`` -> action_13 lexbuf
 |  `\\` -> state_84 lexbuf
 |  `\000` -> backtrack lexbuf
 |  _ -> state_83 lexbuf
and state_83 lexbuf =
  match get_next_char lexbuf with
    `\`` -> action_14 lexbuf
 |  _ -> backtrack lexbuf
and state_84 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_86 lexbuf
 |  `t`|`r`|`n`|`b`|`\``|`\\` -> state_87 lexbuf
 |  _ -> backtrack lexbuf
and state_86 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_89 lexbuf
 |  _ -> backtrack lexbuf
and state_87 lexbuf =
  match get_next_char lexbuf with
    `\`` -> action_15 lexbuf
 |  _ -> backtrack lexbuf
and state_89 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_90 lexbuf
 |  _ -> backtrack lexbuf
and state_90 lexbuf =
  match get_next_char lexbuf with
    `\`` -> action_16 lexbuf
 |  _ -> backtrack lexbuf
and state_96 lexbuf =
  match get_next_char lexbuf with
    `\`` -> action_6 lexbuf
 |  `\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_97 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_99 lexbuf
 |  `t`|`r`|`n`|`b`|`\\` -> state_100 lexbuf
 |  `\`` -> state_101 lexbuf
 |  `\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_98 lexbuf =
  match get_next_char lexbuf with
    `\``|`\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_99 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_104 lexbuf
 |  `\``|`\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_100 lexbuf =
  match get_next_char lexbuf with
    `\`` -> action_7 lexbuf
 |  `\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_101 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_9;
  match get_next_char lexbuf with
    `\`` -> action_7 lexbuf
 |  _ -> backtrack lexbuf
and state_104 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_105 lexbuf
 |  `\``|`\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_105 lexbuf =
  match get_next_char lexbuf with
    `\`` -> action_8 lexbuf
 |  `\000` -> action_9 lexbuf
 |  _ -> state_98 lexbuf
and state_111 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_5;
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_115 lexbuf
 |  `t`|`r`|`n`|`b`|`\\`|`"` -> action_2 lexbuf
 |  `\013` -> state_113 lexbuf
 |  `\010` -> state_112 lexbuf
 |  _ -> backtrack lexbuf
and state_112 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_1;
  match get_next_char lexbuf with
    ` `|`\t` -> state_112 lexbuf
 |  _ -> backtrack lexbuf
and state_113 lexbuf =
  lexbuf.lex_last_pos <- lexbuf.lex_curr_pos;
  lexbuf.lex_last_action <- magic action_1;
  match get_next_char lexbuf with
    ` `|`\010`|`\t` -> state_112 lexbuf
 |  _ -> backtrack lexbuf
and state_115 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> state_116 lexbuf
 |  _ -> backtrack lexbuf
and state_116 lexbuf =
  match get_next_char lexbuf with
    `9`|`8`|`7`|`6`|`5`|`4`|`3`|`2`|`1`|`0` -> action_3 lexbuf
 |  _ -> backtrack lexbuf
and main lexbuf =
  start_lexing lexbuf;
  state_3 lexbuf

and comment lexbuf =
  start_lexing lexbuf;
  state_2 lexbuf

and char lexbuf =
  start_lexing lexbuf;
  state_1 lexbuf

and string lexbuf =
  start_lexing lexbuf;
  state_0 lexbuf
;;
