(* Palm OS Events *)
#open "ref";;
#open "palm";;

(*
 * Event handlers: main handler and form event handlers
 *)

let mainhandler = ref (fun _ -> false)
;;
let event_callbacks = (hashtbl__new 1 : (int, eventHandler) hashtbl__t)
;;

(* THIS FUNCTION MUST HAVE AN EMPTY CLOSURE *)
let dispatch_event_callback (id, evt) =
  match id with
    0 -> !mainhandler evt
  | -1 -> do_at_exit(); false		(* special case *)
  | id -> hashtbl__find event_callbacks id evt
;;

(* The application has to define its own main handler *)
let set_apphandler f =
  mainhandler := f
;;

(* form handlers *)
let set_formhandler id f =
  hashtbl__add event_callbacks id f;
  use_formhandler id
;;

(* iterate on form handlers : we probably have to use that for
 * implementing FrmSaveAllForms instead of going through the PalmOS
 * FrmSaveAllForms (since there is not formID in the frmSaveEvent, we
 * just cannot use the normal callback system)
 *)
let allform_handlers e =
  hashtbl__do_table (fun id f -> f e; ()) event_callbacks 
;;

(*
 * TblDraw callbacks
 *)

let tbldraw_callbacks = (hashtbl__new 1 : ((table*int),tblDraw) hashtbl__t)
;;

let dispatch_tbldraw_callback (table,row,column,rect) =
  (hashtbl__find tbldraw_callbacks (table,column)) row column rect
;;

let set_tbldraw table col f =
  hashtbl__add tbldraw_callbacks (table,col) f;
  use_tabledraw table
;;

(* We have different kinds of callbacks
 *  - event callbacks: event -> bool
 * the order of constructors  in this type has to be kept in sync
 * with the code of clpalm_install_callback in clPalmEvents.c
 *)
let _ =
  install_callback_handler (EventCB dispatch_event_callback);
  install_callback_handler (TblDrawCB dispatch_tbldraw_callback)
;;

(*
 * Debugging (not included in the bytecode if you don't use it !)
 *)
let print_event printer e =
  printer (match e with
  | AppStopEvent -> "appStop"
  | CtlEnterEvent _ -> "ctlEnter"
  | CtlExitEvent _ -> "ctlExit"
  | CtlRepeatEvent _ -> "ctlRepeat"
  | CtlSelectEvent _ -> "ctlSelect"
  | DaySelectEvent _ -> "daySelect"
  | FldChangedEvent _ -> "fldChanged"
  | FldEnterEvent _ -> "fldEnter"
  | FldHeightChangedEvent _ -> "fldHeightChanged"
  | FrmCloseEvent _ -> "frmClose"
  | FrmLoadEvent _ -> "frmLoad"
  | FrmOpenEvent _ -> "frmOpen"
  | FrmGotoEvent _ -> "frmGoto"
  | FrmSaveEvent -> "frmSave"
  | FrmUpdateEvent _ -> "frmUpdate"
  | FrmTitleEnterEvent _ -> "frmTitleEnter"
  | FrmTitleSelectEvent _ -> "frmTitleSelect"
  | KeyDownEvent _ -> "keyDown"
  | LstEnterEvent _ -> "lstEnter"
  | LstExitEvent _ -> "lstExit"
  | LstSelectEvent _ -> "lstSelect"
  | MenuEvent _ -> "menuEvent"
  | NilEvent -> "nilEvent"
  | PenDownEvent _ -> "penDown"
  | PenMoveEvent _ -> "penMove"
  | PenUpEvent _ -> "penUp"
  | PopSelectEvent _ -> "popSelect"
  | SclEnterEvent _ -> "sclEnter"
  | SclRepeatEvent _ -> "sclRepeat"
  | SclExitEvent _ -> "sclExit"
  | TblEnterEvent _ -> "tblEnter"
  | TblExitEvent _ -> "tblExit"
  | TblSelectEvent _ -> "tblSelect"
  | WinEnterEvent _ -> "winEnterEvent"
  | WinExitEvent _ -> "winExitEvent"
)
;;

