#open "palm";;

(* Palm OS Events *)
type modifier =
    Shift | CapsLock | NumLock | Command | Option | Control
  | AutoRepeat | DoubleTap | PoweredOn
;;

type key = {
    chr : int;				(* or char ? *)
    code : int;
    modifiers : int
  } 
;;

type t =
    AppStopEvent

  | CtlEnterEvent of rsrcId * control	(* controlId, pControl *)
  | CtlExitEvent of rsrcId * control	(* controlId, pControl *)
  | CtlRepeatEvent of rsrcId * control * ticks (* controlId, pControl, time *)
  | CtlSelectEvent of rsrcId * control * bool  (* controlId, pControl, on *)

  | DaySelectEvent of dayselector * bool (* pSelector, _, useThisDate *)

  | FldChangedEvent of rsrcId * field	(* fieldID, pField*)
  | FldEnterEvent of rsrcId * field	        (* fieldID, pField*)
  | FldHeightChangedEvent of rsrcId * field * int * int 
	  (* fileId, pField, newHeight, currentPos *)

  | FrmCloseEvent of rsrcId			(* formID *)
  | FrmLoadEvent of rsrcId			(* formID *)
  | FrmOpenEvent of rsrcId			(* formID *)
  | FrmGotoEvent of rsrcId			(* formID *)
  | FrmSaveEvent
  | FrmUpdateEvent of rsrcId * int		(* formId, updateCode *)
  | FrmTitleEnterEvent of rsrcId		(* formId *)
  | FrmTitleSelectEvent of rsrcId		(* formId *)

  | KeyDownEvent of key

  | LstEnterEvent of rsrcId * lst * int	(* listID, pList, selection *)
  | LstExitEvent of rsrcId * lst            (* listID, pList *)
  | LstSelectEvent of rsrcId * lst * int	(* listID, pList, selection *)

  | MenuEvent of rsrcId			(* itemID *)

  | NilEvent

  | PenDownEvent of point		(* _, screenX, screenY *)
  | PenMoveEvent of point		(* _, screenX, screenY *)
  | PenUpEvent of point * point	(* _, screenX, screenY, start, end *)

  | PopSelectEvent of rsrcId * control * rsrcId * lst * int * int
	 (* controlId, pControl, listID, pList, selection, priorSelection *)

  | SclEnterEvent of rsrcId * scrollbar
  | SclRepeatEvent of rsrcId * scrollbar * int * int * ticks
	(* scrollbarId, pScrollBar, value, newValue, time *)
  | SclExitEvent of rsrcId * scrollbar * int * int 
	(* scrollbarId, pScrollBar, value, newValue *)
        
  | TblEnterEvent of rsrcId * table * int * int
  | TblExitEvent of rsrcId * table * int * int
	(* tableID, pTable, row, column *)
  | TblSelectEvent of rsrcId * table * int * int
	(* tableID, pTable, row, column *)

  | WinEnterEvent of win * win (* enterWindow, exitWindow *)
  | WinExitEvent of win * win (* enterWindow, exitWindow *)
;;

type eventHandler == t -> bool;;
value set_apphandler : eventHandler -> unit;;
value set_formhandler : rsrcId -> eventHandler -> unit;;

type tblDraw == int -> int -> (point * point) -> unit;;

(*-*)
value define_break_event : t -> unit = 1 "clpalm_define_break_event";;
value print_event : (string -> unit) -> t -> unit;;


(* sum types of all callbacks : each callback function may have only
 * one argument !
 *)
type callbacks =
    EventCB of (int * t -> bool)  (* [handle event] *)
  | TblDrawCB of (table * int * int * (point * point) -> unit)
	(* [tbldraw tbl row column (point, extent)] *)
  | TblLoadCB of (table * int * int * bool * field) -> (handle * int * int)
	(* [tblload tbl row col editable] returns handle, offset, size *)
  | TblSaveCB of (table * int * int -> bool)
	(* [tblsave tbl row col] *)
;;
value install_callback_handler : callbacks -> unit = 1 "clpalm_install_callback";;

value use_formhandler : int -> unit = 1 "clpalm_useformhandler";;
value use_tabledraw : table -> int -> unit = 2 "clpalm_settbldraw";;
value use_tableload : table -> int -> unit = 2 "clpalm_settblload";;
