#include "mlvalues.h"
#include "fail.h"
#include "sys.h"
#include "memory.h"
#include "alloc.h"
#include "str.h"
#include "clPalm_cvt.h"

/* Dm stuff */
value clpalm_dmArchiveRecord (value dm, value index)	/* ML */
{
  Err err = DmArchiveRecord((DmOpenRef)dm, Int_val(index));
  if (err == 0) return Val_unit; else sys_error(err);
}

/* DmAttachRecord: needs handle */
/* DmAttachResource: needs handle */

value clpalm_dmCloseDatabase (value dm)	/* ML */
{
  Err err = DmCloseDatabase((DmOpenRef)dm);
  if (err == 0) return Val_unit; else sys_error(err);
}

value clpalm_dmCreateDatabase (value cardno, value name, value creator,
			       value type, value res) /* ML */
{
  Err err = DmCreateDatabase(Int_val(cardno), String_val(name),
			     id_of_string(creator), id_of_string(type),
			     Bool_val(res));
  if (err == 0) return Val_unit; else sys_error(err);
}

/* DmCreateDatabaseFromImage: requires ptr to large string */

value clpalm_dmDatabaseInfo (value cardno, value dbid)	/* ML */
{
  char name[32]; /* TODO: check max size in doc */
  UInt attributes;
  UInt version;
  ULong crdate;
  ULong moddate;
  ULong bckupdate;
  ULong modnum;
  LocalID appInfo;
  LocalID sortInfo;
  ULong type;
  ULong creator;
  Err err;
  value res;

  Push_roots(r, 5);

  err = DmDatabaseInfo(Int_val(cardno), id_of_string(dbid),
		       &name[0], &attributes, &version, &crdate, &moddate,
		       &bckupdate, &modnum, &appInfo, &sortInfo, &type,
		       &creator);

  r[0] = copy_string(name);
  r[1] = string_of_id(appInfo);
  r[2] = string_of_id(sortInfo);
  r[3] = string_of_id(type);
  r[4] = string_of_id(creator);

  res = alloc_tuple(11);
  Field(res, 0) = r[0];
  Field(res, 1) = Val_int(attributes);	/* TODO : check upper bit */
  Field(res, 2) = Val_int(version);
  Field(res, 3) = Val_int(crdate);
  Field(res, 4) = Val_int(moddate);
  Field(res, 5) = Val_int(bckupdate);
  Field(res, 6) = Val_int(modnum);
  Field(res, 7) = r[1];
  Field(res, 8) = r[2];
  Field(res, 9) = r[3];
  Field(res, 10) = r[4];
  Pop_roots();
  return res;
}

value clpalm_dmDatabaseProtect (value cardno, value localid, value protect) /* ML */
{
  Err err = DmDatabaseProtect(Int_val(cardno), id_of_string(localid), 
			      Bool_val(protect));
  if (err == 0) return Val_unit; else sys_error(err);
}

value clpalm_dmDatabaseSize (value cardno, value dbid)	/* ML */
{
  ULong numrecords;
  ULong totalBytes;
  ULong dataBytes;

  Err err;

  err = DmDatabaseSize(Int_val(cardno), id_of_string(dbid),
		       &numrecords, &totalBytes, &dataBytes);
  if (err == 0) {
    value res = alloc_tuple(3);
    Field(res, 0) = Val_int(numrecords);
    Field(res, 1) = Val_int(totalBytes);
    Field(res, 2) = Val_int(dataBytes);
    return res;
    } 
  else sys_error(err);
}

value clpalm_dmDeleteCategory (value dm, value cnum) /* ML */
{
  Err err = DmDeleteCategory((DmOpenRef) dm, Int_val(cnum));
  if (err == 0) return Val_unit; else sys_error(err);
}

value clpalm_dmDeleteDatabase (value cardno, value id) /* ML */
{
  Err err = DmDeleteDatabase(Int_val(cardno), id_of_string(id));
  if (err == 0) return Val_unit; else sys_error(err);
}

value clpalm_dmDeleteRecord (value dm, value idx)	/* ML */
{
  Err err = DmDeleteRecord((DmOpenRef)dm, Int_val(idx));
  if (err == 0) return Val_unit; else sys_error(err);
}

/* DmDetachRecord requires handle */
/* DmDetachResource requires handle */
value clpalm_dmFindDatabase (value cardno, value name)	/* ML */
{
  LocalID id = DmFindDatabase(Int_val(cardno), String_val(name));
  if (id == 0) mlraise(NOT_FOUND_EXN);
  else return(string_of_id (id));
}

value clpalm_dmFindRecordByID (value dm, value id) /* ML */
{
  Err err;
  UInt idx;
  err = DmFindRecordByID((DmOpenRef) dm, id_of_string(id), &idx);
  if (err == 0) return Val_int(idx);
  else sys_error(err);
}

/* DmFindResource requires handle */

value clpalm_dmFindResourceType (value dm, value restype, value start) /* ML */
{
  Int idx = DmFindResourceType((DmOpenRef) dm, id_of_string(restype), 
			       Int_val(start));
  if (idx == -1) mlraise(NOT_FOUND_EXN);
  else return Val_int(idx);
}

/* DmFindSortPosition : requires pointers */

value clpalm_dmGetAppInfoID (value dm)	/* ML */
{
  return string_of_id(DmGetAppInfoID((DmOpenRef)dm));
}

value clpalm_dmGetDatabase (value cardno, value idx)	/* ML */
{
  return string_of_id(DmGetDatabase(Int_val(cardno), Int_val(idx)));
}

/* DmGetLastErr : probably not needed */

/* DmGetNextDatabaseByTypeCreator : we have to find the proper interface */

