#include "config.h"
#include "mlvalues.h"
#include "fail.h"
#include "alloc.h"
#include "memory.h"
#include "ui.h"
#include "clPalm_cvt.h"
extern value callback(value closure, value argument);

/* Convert an Event into its ML value */

/* CHECK: this must be in the same order as constructors in events.mli */
enum mlevents {
  MLAppStop,
  MLCtlEnter, MLCtlExit, MLCtlRepeat, MLCtlSelect,
  MLDaySelect,
  MLFldChanged, MLFldEnter, MLFldHeightChanged,
  MLFrmClose, MLFrmLoad, MLFrmOpen, MLFrmGoto, MLFrmSave, MLFrmUpdate,
  MLFrmTitleEnter, MLFrmTitleSelect,
  MLKeyDown,
  MLLstEnter, MLLstExit, MLLstSelect,
  MLMenu,
  MLNil,
  MLPenDown, MLPenMove, MLPenUp,
  MLPopSelect,
  MLSclEnter, MLSclRepeat, MLSclExit,
  MLTblEnter, MLTblExit, MLTblSelect,
  MLWinEnter, MLWinExit
};


value clpalm_event2ml(EventPtr event)
{
  tag_t tag;
  value res;

  switch(event->eType) {

  case penDownEvent:
    tag = MLPenDown; goto penfill;
  case penMoveEvent:
    tag = MLPenMove;
  penfill:
    { Push_roots(r,1);
      r[0] = alloc_tuple(2); 
      Field(r[0],0) = Val_int(event->screenX);
      Field(r[0],1) = Val_int(event->screenY);
      res = alloc(1, tag);
      Field(res, 0) = r[0];
      Pop_roots();
      break;
    }

  case penUpEvent:
    { Push_roots(r,2);
      r[0] = alloc_tuple(2);
      Field(r[0],0) = Val_int(event->data.penUp.start.x);
      Field(r[0],1) = Val_int(event->data.penUp.start.y);
      r[1] = alloc_tuple(2);
      Field(r[1],0) = Val_int(event->data.penUp.end.x);
      Field(r[1],1) = Val_int(event->data.penUp.end.y);
      res = alloc(2,MLPenUp);
      Field(res,0) = r[0];
      Field(res,1) = r[1];
      Pop_roots();
      break;
    }

  case keyDownEvent:
    { Push_roots(r,1);
      r[0] = alloc_tuple(3);
      Field(r[0],0) = Val_int(event->data.keyDown.chr);
      Field(r[0],1) = Val_int(event->data.keyDown.keyCode);
      Field(r[0],2) = Val_int(event->data.keyDown.modifiers);
      res = alloc(1, MLKeyDown);
      Field(res, 0) = r[0];
      Pop_roots();
      break;
    }

  case winEnterEvent:
    tag = MLWinEnter; goto winfill;
  case winExitEvent:
    tag = MLWinExit;
  winfill:
    res = alloc(2, tag);
    Field(res, 0) = (value)(event->data.winEnter.enterWindow);
    Field(res, 1) = (value)(event->data.winEnter.exitWindow);
    break;

  case ctlEnterEvent:
    res = alloc(2, MLCtlEnter); goto ctlfill;
  case ctlExitEvent:
    res = alloc(2, MLCtlExit); goto ctlfill;
  case ctlSelectEvent:
    res = alloc(3, MLCtlSelect);
    Field(res, 2) = event->data.ctlSelect.on ? Val_true : Val_false;
    goto ctlfill;
  case ctlRepeatEvent:
    res = alloc(3, MLCtlRepeat);
    Field(res, 2) = Val_int(event->data.ctlRepeat.time);
  ctlfill:
    Field(res, 0) = Val_int(event->data.ctlRepeat.controlID);
    Field(res, 1) = (value)(event->data.ctlRepeat.pControl);
    break;

  case lstEnterEvent:
    res = alloc(3, MLLstEnter); goto lstfill1;
  case lstSelectEvent:
    res = alloc(3, MLLstSelect); 
  lstfill1:
    Field(res, 2) = Val_int(event->data.lstEnter.selection);
    goto lstfill;
  case lstExitEvent:
    res = alloc(2, MLLstExit);
  lstfill:
    Field(res, 0) = Val_int(event->data.lstExit.listID);
    Field(res, 1) = (value)(event->data.lstExit.pList);
    break;

  case popSelectEvent:
    res = alloc(6, MLPopSelect);
    Field(res, 0) = Val_int(event->data.popSelect.controlID);
    Field(res, 1) = (value)(event->data.popSelect.controlP);
    Field(res, 2) = Val_int(event->data.popSelect.listID);
    Field(res, 3) = (value)(event->data.popSelect.listP);
    Field(res, 4) = Val_int(event->data.popSelect.selection);
    Field(res, 5) = Val_int(event->data.popSelect.priorSelection);
    break;

  case fldEnterEvent:
    res = alloc(2, MLFldEnter); goto fldfill;
  case fldChangedEvent:
    res = alloc(2, MLFldChanged); goto fldfill;
    /*
  case fldExitEvent:
    res = alloc(2, MLFldExit); goto fldfill;
    */
  case fldHeightChangedEvent:
    res = alloc(4, MLFldHeightChanged);
    Field(res, 2) = Val_int(event->data.fldHeightChanged.newHeight);
    Field(res, 3) = Val_int(event->data.fldHeightChanged.currentPos);
  fldfill:
    Field(res, 0) = Val_int(event->data.fldHeightChanged.fieldID);
    Field(res, 1) = (value)(event->data.fldHeightChanged.pField);
    break;

  case tblEnterEvent:
    tag = MLTblEnter; goto tblfill;
  case tblSelectEvent:
    tag = MLTblSelect; goto tblfill;
  case tblExitEvent:
    tag = MLTblExit;
  tblfill:
    res = alloc(4, tag);
    Field(res, 0) = Val_int(event->data.tblEnter.tableID);
    Field(res, 1) = (value)(event->data.tblEnter.pTable);
    Field(res, 2) = Val_int(event->data.tblEnter.row);
    Field(res, 3) = Val_int(event->data.tblEnter.column);
    break;

  case daySelectEvent:
    res = alloc(2, MLDaySelect);
    Field(res, 0) = (value)(event->data.daySelect.pSelector);
    Field(res, 1) = event->data.daySelect.useThisDate ? Val_true : Val_false;
    break;
    
  case menuEvent:
    res = alloc(1, MLMenu);
    Field(res, 0) = Val_int(event->data.menu.itemID);
    break;

  case appStopEvent:
    res = Atom(MLAppStop);
    break;

  /* compact code (assuming same layout for these events)  */
  case frmLoadEvent:
    tag = MLFrmLoad; goto frmfill;
  case frmOpenEvent:
    tag = MLFrmOpen; goto frmfill;
  case frmGotoEvent:   /* this one doesn't appear in the documentation ! */
    tag = MLFrmGoto; goto frmfill;
  case frmCloseEvent:
    tag = MLFrmClose; goto frmfill;
  case frmTitleEnterEvent:
    tag = MLFrmTitleEnter; goto frmfill;
  case frmTitleSelectEvent:
    tag = MLFrmTitleSelect;
  frmfill:
    res = alloc(1, tag);
    Field(res, 0) = Val_int(event->data.frmLoad.formID);
    break;

  case frmSaveEvent:
    res = Atom(MLFrmSave);
    break;

  case frmUpdateEvent:
    res = alloc(2, MLFrmUpdate);
    Field(res, 0) = Val_int(event->data.frmUpdate.formID);
    Field(res, 1) = Val_int(event->data.frmUpdate.updateCode);
    break;

  case sclEnterEvent:
    res = alloc(2, MLSclEnter); goto sclfill;
  case sclExitEvent:
    res = alloc(4, MLSclExit);
    Field(res, 2) = Val_int(event->data.sclExit.value);
    Field(res, 3) = Val_int(event->data.sclExit.newValue);
    goto sclfill;
  case sclRepeatEvent:
    res = alloc(5, MLSclRepeat);
    Field(res, 2) = Val_int(event->data.sclRepeat.value);
    Field(res, 3) = Val_int(event->data.sclRepeat.newValue);
    Field(res, 4) = Val_int(event->data.sclRepeat.time);
  sclfill:
    Field(res, 0) = Val_int(event->data.sclRepeat.scrollBarID);
    Field(res, 1) = (value)(event->data.sclRepeat.pScrollBar);
    break;

  case nilEvent:
  default: /* ??? */
    res = Atom(MLNil);	
    break;
  }

  return res;
}

/* Given the ML event, fill an Event with the documented fields */
/* Mapping from ML constructor to Event enum 
 * In the order of the constructors 
 */
static enum events ml2c[] = {
  appStopEvent, 
  ctlEnterEvent, ctlExitEvent, ctlRepeatEvent, ctlSelectEvent,
  daySelectEvent,
  fldChangedEvent, fldEnterEvent, fldHeightChangedEvent,
  frmCloseEvent, frmLoadEvent, frmOpenEvent, frmGotoEvent, frmSaveEvent, frmUpdateEvent,
  keyDownEvent,
  lstEnterEvent, lstExitEvent, lstSelectEvent,
  menuEvent,
  nilEvent,
  penDownEvent, penMoveEvent, penUpEvent,
  popSelectEvent,
  sclEnterEvent, sclRepeatEvent, sclExitEvent,
  tblEnterEvent, tblExitEvent, tblSelectEvent,
  winEnterEvent, winExitEvent
};

void clpalm_mlevent2c(value event, EventPtr e)
{
  bzero(e, sizeof(EventType));
  e->eType = ml2c[Tag_val(event)];

  switch(Tag_val(event)) {
  case MLAppStop:
    break;

  /* NOTE: code sharing assumes layout DOES NOT CHANGE */
  case MLCtlRepeat:
    e->data.ctlRepeat.time = Int_val(Field(event,2));
    goto fillctl;
  case MLCtlSelect:
    e->data.ctlSelect.on = Val_bool(Field(event,2));
    goto fillctl;
  case MLCtlExit:
  case MLCtlEnter:
  fillctl:
    e->data.ctlEnter.controlID = Int_val(Field(event,0));
    e->data.ctlEnter.pControl = (ControlPtr)(Field(event,1));
    break;
    
  case MLDaySelect:
    e->data.daySelect.pSelector = (void *)Field(event,0);
    e->data.daySelect.useThisDate = Val_bool(Field(event,1));
    break;
    
  case MLFldHeightChanged:
    e->data.fldHeightChanged.newHeight = Int_val(Field(event,2));
    e->data.fldHeightChanged.currentPos = Int_val(Field(event,3));
    /* fall through */
  case MLFldChanged:
  case MLFldEnter:
    e->data.fldEnter.fieldID = Int_val(Field(event,0));
    e->data.fldEnter.pField = (FieldPtr)(Field(event,1));
    break;

  case MLFrmUpdate:
    e->data.frmUpdate.updateCode = Int_val(Field(event,1));
    /* fall through */
  case MLFrmClose:
  case MLFrmLoad:
  case MLFrmOpen:
  case MLFrmGoto:
  /* this one doesn't appear in the documentation */
  /* WARNING: we don't know the other fields ! */
  case MLFrmTitleEnter:
  case MLFrmTitleSelect:
    e->data.frmLoad.formID = Int_val(Field(event,0));
    break;

  case MLFrmSave:
    break;
    
  case MLKeyDown:
    {
      value k = Field(event,0);
      e->data.keyDown.chr = Int_val(Field(k, 0));
      e->data.keyDown.keyCode = Int_val(Field(k, 1));
      e->data.keyDown.modifiers = Int_val(Field(k, 2));
      break;
    }
    
  case MLLstSelect:
    e->data.lstSelect.selection = Int_val(Field(event,2));
    /* fall through */
  case MLLstEnter:
  case MLLstExit:
    e->data.lstEnter.listID = Int_val(Field(event,0));
    e->data.lstEnter.pList = (ListPtr)(Field(event,1));
    break;
    
  case MLMenu:
    e->data.menu.itemID = Int_val(Field(event,0));
    break;

  case MLNil:
    break;
    
  case MLPenDown:
  case MLPenMove:
    {
      value point = Field(event, 0);
      e->penDown = true;
      e->screenX = Int_val(Field(point,0));
      e->screenY = Int_val(Field(point,1));
      break;
    }

  case MLPenUp:
    { value point;
      e->penDown = false;
      point = Field(event,0);
      e->data.penUp.start.x = Int_val(Field(point,0));
      e->data.penUp.start.y = Int_val(Field(point,1));
      point = Field(event,1);
      e->data.penUp.end.x = Int_val(Field(point,0));
      e->data.penUp.end.y = Int_val(Field(point,1));
      break;
    }
    
  case MLPopSelect:
    e->data.popSelect.controlID = Int_val(Field(event,0));
    e->data.popSelect.controlP = (ControlPtr)(Field(event,1));
    e->data.popSelect.listID = Int_val(Field(event,2));
    e->data.popSelect.listP = (void *)(Field(event,3));
    e->data.popSelect.selection = Int_val(Field(event,4));
    e->data.popSelect.priorSelection = Int_val(Field(event,5));
    break;

  case MLSclRepeat:
    e->data.sclRepeat.value = Int_val(Field(event,2));
    e->data.sclRepeat.newValue = Int_val(Field(event,3));
    e->data.sclRepeat.time = Int_val(Field(event,4));
    /* fall through */
  case MLSclEnter:
  case MLSclExit:
    e->data.sclEnter.scrollBarID = Int_val(Field(event,0));
    e->data.sclEnter.pScrollBar = (void *)(Field(event,1));
    break;
    
  case MLTblEnter:
  case MLTblExit:
  case MLTblSelect:
    e->data.tblEnter.tableID = Int_val(Field(event,0));
    e->data.tblEnter.pTable = (void *)(Field(event,1));
    e->data.tblEnter.row = Int_val(Field(event,2));
    e->data.tblEnter.column = Int_val(Field(event,3));
    break;
    
  case MLWinEnter:
  case MLWinExit:
    e->data.winEnter.enterWindow = (void *)(Field(event,0));
    e->data.winEnter.exitWindow = (void *)(Field(event,1));
    break;
  }

}


/* Code part of the event callback dispatcher */
static code_t event_handler_code = NULL;
static code_t tbl_draw_code = NULL;
static code_t tbl_load_code = NULL;
static code_t tbl_save_code = NULL;
static code_t lst_draw_code = NULL;
/* Initialization: has to be called with the GLOBAL callback dispatcher,
 * which closure is empty.
 */
value clpalm_install_callback(handler) /* ML */
  value handler;
{
  switch (Tag_val(handler)) {
  case 0:
    event_handler_code = Code_val(Field(handler,0));
    break;
  case 1:
    tbl_draw_code = Code_val(Field(handler,0));
    break;
  case 2:
    tbl_load_code = Code_val(Field(handler,0));
    break;
  case 3:
    tbl_save_code = Code_val(Field(handler,0));
    break;
  case 4:
    lst_draw_code = Code_val(Field(handler, 0));
    break;
  }
  return Atom(0);
}

/* Hack for calling Caml global function from C */
/* arg has been allocated by us, and it might move
   when we allocate the closure
   */
value clpalm_callback(code_t handler_code, value arg)
{
  value clos;
  value res;
  Push_roots(r,1);
#define garg r[0]
  garg = arg;
  clos = alloc(2,Closure_tag);
  Code_val(clos) = handler_code;
  Env_val(clos) = Atom(0);	/* empty closure */
  res = callback(clos, garg);
  Pop_roots();
#undef garg
  return res;
}

/* First wrapper: takes an int (form ID) and a value (event) */
value clpalm_dispatch_event_callback(int id, value arg)
{
  value res;
  value pair;
  Push_roots(r,1);
#define garg r[0]
  garg = arg;			/* save it */
  pair = alloc_tuple(2);
  Field(pair, 0) = Val_int(id);
  Field(pair, 1) = garg;
  res = clpalm_callback(event_handler_code, pair);
  Pop_roots();
#undef garg
  return res;
}

/* In the periodic action, we check for the break event (only one...) */
static EventType breakEvent;

/* Store that event definition */
value clpalm_define_break_event(value event) /* ML */
{
  clpalm_mlevent2c(event, &breakEvent);
  return Val_unit;
}

/* We check if this is the Break event, and otherwise call the
 * application handler (number 0 by convention).
 */

Boolean ApplicationHandleEvent(EventPtr event)
{
  if ((event->eType == breakEvent.eType) 
      && (event->data.generic.data1 == breakEvent.data.generic.data1))
    mlraise(BREAK_EXN);
  else {
    value mlevent = clpalm_event2ml(event);
    value ret = clpalm_dispatch_event_callback(0, mlevent);
    /* the at_exit stuff in the normal case */
    if (event->eType == appStopEvent)
      clpalm_dispatch_event_callback(-1,clpalm_event2ml(event));
    return Bool_val(ret);
  }
}

/* What we must do periodically is to handle system events and menu events,
 * if there are any.
 */
#ifdef 0
void ui_periodic_action()
{
  EventType event;
  Err err;

  EvtGetEvent(&event, 0);
  if (SysHandleEvent(&event)) return; /* the system handles it */
  if (MenuHandleEvent(0, &event, &err)) return; /* the menu handles it */
  if ((event.eType == breakEvent.eType) 
      && (event.data.generic.data1 == breakEvent.data.generic.data1))
    /* raise Break */;
  else { /* in a normal event loop, we process it
	  * in this case, we just put it back.
	  * This disrupts the queue order, but we lack the feature to
	  * copy the queue while handling only Sys/Menu events.
	  */
    EvtAddEventToQueue(&event);
  }
}
#endif

/* Form event handlers.
 * there is no closure support in PalmOS,
 * and no support in Caml Light for calling arbitrary closures from C.
 * This means that we have only a unique possible FormHandler that can
 * be used as argument of FrmSetEventHandler.
 * The question is: how do we call the appropriate handler ?
 *  if it's a frm*Event with a formID we dispatch to this form
 *  else, we use the active form (FrmGetActiveFormID)
 * NOTE: there is a problem with frmSave events. The solution is probably
 * to handle them in the main application handler (iterate on form handlers
 * in the hashtbl).
 */

/* This is the one that we must pass to PalmOS, for *all* forms */
/* the %a4 stuff is required for prc-tools <= 0.5.0 */

register void *reg_a4 asm("%a4");

#ifdef 0
This code works only for main application, but in the case of GLib shared
libraries, it should be replaced by something else.
#define CALLBACK_PROLOGUE \
    void *save_a4 = reg_a4; asm("move.l %%a5,%%a4; sub.l #edata,%%a4" : :)

#define CALLBACK_EPILOGUE reg_a4 = save_a4
#endif

struct LibRef {
    void *jmptable;
    void *globals;
};

#define CALLBACK_PROLOGUE \
    void *save_a4 = reg_a4; \
    Err err; \
    struct LibRef *libref; \
    err = FtrGet('CLPR', 0, (DWord *)&libref); \
    reg_a4 = libref->globals;

#define CALLBACK_EPILOGUE reg_a4 = save_a4


static Boolean DispatchFormHandler(EventPtr event)
{
  Word id;
  Boolean res;
  // printf("generic form handler %d\n", event->eType);
  CALLBACK_PROLOGUE;
  switch (event->eType) {
  case frmLoadEvent:
  case frmOpenEvent:
  case frmGotoEvent:
  case frmCloseEvent:
    id = event->data.frmLoad.formID;
    break;
  default:
    id = FrmGetActiveFormID();
    break;
  }
  res = Bool_val(clpalm_dispatch_event_callback(id, clpalm_event2ml(event)));
  CALLBACK_EPILOGUE;
  return res;
}


value clpalm_useformhandler (value formID) /* ML */
{
  FormPtr fptr = FrmGetFormPtr(Int_val(formID));
  if (fptr)
    FrmSetEventHandler (fptr, DispatchFormHandler);
  else 
    raise_with_string(INVALID_EXN, "form not loaded");
  return Val_unit;
}

/* Table drawing
 * This has type TableDrawItemFuncType
 * Since it has a table argument, we can use it as key to dispatch back to
 * a hashtbl in ML
 */
static void DispatchTblDraw(VoidPtr table, Word row, Word column, RectanglePtr bounds)
{
  CALLBACK_PROLOGUE;
  { value arg;
    Push_roots(r,3);
    r[0] = alloc_tuple(2);
    Field(r[0], 0) = Val_int(bounds->topLeft.x);
    Field(r[0], 1) = Val_int(bounds->topLeft.y);
    r[1] = alloc_tuple(2);
    Field(r[1], 0) = Val_int(bounds->extent.x);
    Field(r[1], 1) = Val_int(bounds->extent.y);
    r[2] = alloc_tuple(2);
    Field(r[2], 0) = r[0];
    Field(r[2], 1) = r[1];
    arg = alloc_tuple(4);
    Field(arg,0) = (value)table;
    Field(arg,1) = Val_int(row);
    Field(arg,2) = Val_int(column);
    Field(arg,3) = r[2];
    clpalm_callback(tbl_draw_code, arg);
    Pop_roots();
  }
  CALLBACK_EPILOGUE;
}

value clpalm_settabledraw (value tbl, value col) /* ML */
{
  TblSetCustomDrawProcedure((TablePtr)tbl, Int_val(col), DispatchTblDraw);
  return Val_unit;
}


/* Table loading 
 * same thing (doc is not up to date...)
 */
static Err DispatchLoadData(VoidPtr table, Word row, Word column, Boolean editable,
		     VoidHand *dh, WordPtr dataOffset, WordPtr dataSize, FieldPtr fld)
{
  value arg;
  value res;
  CALLBACK_PROLOGUE;
  arg = alloc_tuple(5);
  Field(arg, 0) = (value)table;
  Field(arg, 1) = Val_int(row);
  Field(arg, 2) = Val_int(column);
  Field(arg, 3) = Val_bool(editable);
  Field(arg, 4) = (value)fld;
  res = clpalm_callback(tbl_load_code, arg);
  *dh = (VoidHand)Field(res, 0);
  *dataOffset = Int_val(Field(res, 1));
  *dataSize =  Int_val(Field(res, 2));
  CALLBACK_EPILOGUE;
  return 0;
}

value clpalm_settblload (value tbl, value col) /* ML */
{
  TblSetLoadDataProcedure((TablePtr)tbl, Int_val(col), DispatchLoadData);
  return Val_unit;
}

/* Table saving */
static Boolean DispatchSaveData(VoidPtr table, Word row, Word column)
{
  value arg, res;
  CALLBACK_PROLOGUE;
  arg = alloc_tuple(3);
  Field(arg, 0) = (value)table;
  Field(arg, 1) = Val_int(row);
  Field(arg, 2) = Val_int(column);
  res = clpalm_callback(tbl_save_code, arg);
  CALLBACK_EPILOGUE;
  return Bool_val(res);
}

value clpalm_settblsave (value tbl, value col) /* ML */
{
  TblSetSaveDataProcedure((TablePtr)tbl, Int_val(col), DispatchSaveData);
  return Val_unit;
}


/* This has type ListDrawDataFuncType 
 * We don't know really how to dispatch to the proper handler in ML, because none
 * of the arguments really indicates on which List we are supposed to work.
 * The solution is probably to use a special convention with the items.
 */
void DispatchListSetDraw(Word itemNum, RectanglePtr bounds, CharPtr *itemsText)
{
  CALLBACK_PROLOGUE;
  { 
    Push_roots(r,2);
    r[0] = rect_c2ml(bounds);
    r[1] = alloc_tuple(3);
    Field(r[1], 0) = Val_int(itemNum);
    Field(r[1], 1) = r[0];
    Field(r[1], 2) = copy_string(itemsText[itemNum]);
    clpalm_callback(lst_draw_code, r[1]);
    Pop_roots();
  }
  CALLBACK_EPILOGUE;
}
