#include "mlvalues.h"
#include "fail.h"
#include "alloc.h"
#include "memory.h"
#include "io.h"
#include "str.h"
#include "clPalm_cvt.h"

/*
 * Field functions
 */

value clpalm_FldCalcFieldHeight(value chars, value width) /* ML */
{
  return Val_int(FldCalcFieldHeight(String_val(chars), Int_val(width)));
}

value clpalm_FldCompactText(value fld) /* ML */
{
  FldCompactText((FieldPtr)fld);
  return Val_unit;
}

/* p214 : fld -> unit */
value clpalm_FldCopy(value fptr) /* ML */
{
  FldCopy((FieldPtr) fptr);
  return Val_unit;
}

/* p214 : fld -> unit */
value clpalm_FldCut(value fptr) /* ML */
{
  FldCut((FieldPtr) fptr);
  return Val_unit;
}

value clpalm_FldDelete(value fptr, value vstart, value vend) /* ML */
{
  FldDelete((FieldPtr)fptr, Int_val(vstart), Int_val(vend));
  return Val_unit;
}

value clpalm_FldDirty(value fptr) /* ML */
{
  return Val_bool(FldDirty((FieldPtr)fptr));
}

/* p216 : fld -> unit */
value clpalm_FldDrawField(value fptr) /* ML */
{
  FldDrawField((FieldPtr)fptr);
  return Val_unit;
}

value clpalm_FldEraseField(value fptr) /* ML */
{
  FldEraseField((FieldPtr)fptr);
  return Val_unit;
}

value clpalm_FldFreeMemory(value fptr) /* ML */
{
  FldFreeMemory((FieldPtr)fptr);
  return Val_unit;
}

value clpalm_FldGetBounds(value fptr) /* ML */
{
  RectangleType rect;
  FldGetBounds((FieldPtr)fptr, &rect);
  return rect_c2ml(&rect);
}

/* SYNC definition of font__id */
value clpalm_FldGetFont(value fptr) /* ML */
{
  FontID id = FldGetFont((FieldPtr)fptr);
  return Atom(id);
}

value clpalm_FldGetInsPtPosition(value ftpr) /* ML */
{
  return Val_int(FldGetInsPtPosition((FieldPtr)ftpr));
}

value clpalm_FldGetMaxChars(value fptr) /* ML */
{
  return Val_int(FldGetMaxChars((FieldPtr)fptr));
}

value clpalm_FldGetNumberOfBlankLines(value fptr) /* ML */
{
  return Val_int(FldGetNumberOfBlankLines((FieldPtr)fptr));
}

value clpalm_FldGetScrollPosition(value fptr) /* ML */
{
  return Val_int(FldGetScrollPosition((FieldPtr)fptr));
}

value clpalm_FldGetScrollValues(value fptr) /* ML */
{
  Word scrollPos, textHeight, fieldHeight;
  value res;
  FldGetScrollValues((FieldPtr)fptr, &scrollPos, &textHeight, &fieldHeight);
  res = alloc_tuple(3);
  Field(res, 0) = Val_int(scrollPos);
  Field(res, 1) = Val_int(textHeight);
  Field(res, 2) = Val_int(fieldHeight);
  return res;
}

value clpalm_FldGetSelection(value fptr) /* ML */
{
  Word startPosition, endPosition; 
  value res;
  FldGetSelection((FieldPtr)fptr, &startPosition, &endPosition);
  res = alloc_tuple(2);
  Field(res, 0) = Val_int(startPosition);
  Field(res, 1) = Val_int(endPosition);
  return res;
}


value clpalm_FldGetTextAllocatedSize(value fptr) /* ML */
{
  return Val_int(FldGetTextAllocatedSize((FieldPtr)fptr));
}

value clpalm_FldGetTextHandle(value fptr) /* ML */
{
  return (value)FldGetTextHandle((FieldPtr)fptr);
}

value clpalm_FldGetTextHeight(value fptr) /* ML */
{
  return Val_int(FldGetTextHeight((FieldPtr)fptr));
}

/* p224 : fld -> int */
value clpalm_FldGetTextLength(value fptr) /* ML */
{
  return Val_int(FldGetTextLength((FieldPtr) fptr));
}


/* fptr -> string 
 * replaces FldGetTextHandle and FldGetTextPtr
 * returns a copy of the string
 */
value clpalm_FldGetText(value fptr) /* ML */
{
  char *ptr = FldGetTextPtr((FieldPtr)fptr);
  if (ptr == NULL) mlraise(Atom(NOT_FOUND_EXN));
  return copy_string(ptr);
}

value clpalm_FldGetVisibleLines(value fptr) /* ML */
{
  return Val_int(FldGetVisibleLines((FieldPtr)fptr));
}

value clpalm_FldGrabFocus(value fptr) /* ML */
{
  FldGrabFocus((FieldPtr)fptr);
  return Val_unit;
}

value clpalm_FldHandleEvent(value fptr, value ev) /* ML */
{
  EventType e;
  clpalm_mlevent2c(ev, &e);
  return Val_bool(FldHandleEvent((FieldPtr)fptr, &e));
}

value clpalm_FldInsert(value fptr, value str, value len) /* ML */
{
  return Val_bool(FldInsert((FieldPtr)fptr, String_val(str), Int_val(len)));
}

value clpalm_FldMakeFullyVisible(value fptr) /* ML */
{
  return Val_bool(FldMakeFullyVisible((FieldPtr)fptr));
}

/* p229 : fld -> unit */
value clpalm_FldPaste(value fptr) /* ML */
{
  FldPaste((FieldPtr) fptr);
  return Val_unit;
}

value clpalm_FldRecalculateField(value fptr, value bool) /* ML */
{
  FldRecalculateField((FieldPtr)fptr, Bool_val(bool));
  return Val_unit;
}

value clpalm_FldReleaseFocus(value fptr) /* ML */
{
  FldReleaseFocus((FieldPtr)fptr);
  return Val_unit;
}

value clpalm_FldScrollable(value fptr, value direction) /* ML */
{
  return Val_bool(FldScrollable((FieldPtr)fptr, Tag_val(direction)));
}

value clpalm_FldScrollField(value fptr, value lines, value direction) /* ML */
{
  FldScrollField((FieldPtr)fptr, Int_val(lines), Tag_val(direction));
  return Val_unit;
}

value clpalm_FldSendChangeNotification(value fptr) /* ML */
{
  FldSendChangeNotification((FieldPtr)fptr);
  return Val_unit;
}

value clpalm_FldSendHeightChangeNotification(value fptr, value pos, value n) /* ML */
{
  FldSendHeightChangeNotification((FieldPtr)fptr, Int_val(pos), Int_val(n));
  return Val_unit;
}

value clpalm_FldSetBounds(value fptr, value vrect) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  FldSetBounds((FieldPtr)fptr, &rect);
  return Val_unit;
}

value clpalm_FldSetDirty(value fptr, value flag) /* ML */
{
  FldSetDirty((FieldPtr)fptr, Bool_val(flag));
  return Val_unit;
}

value clpalm_FldSetFont(value fptr, value fontid) /* ML */
{
  FldSetFont((FieldPtr)fptr, Tag_val(fontid));
  return Val_unit;
}

value clpalm_FldSetInsertionPoint(value fptr, value pos) /* ML */
{
  FldSetInsertionPoint((FieldPtr)fptr, Int_val(pos));
  return Val_unit;
}

value clpalm_FldSetInsPtPosition(value fptr, value pos) /* ML */
{
  FldSetInsPtPosition((FieldPtr)fptr, Int_val(pos));
  return Val_unit;
}

value clpalm_FldSetMaxChars(value fptr, value max) /* ML */
{
  FldSetMaxChars((FieldPtr)fptr, Int_val(max));
  return Val_unit;
}

value clpalm_FldSetScrollPosition(value fptr, value pos) /* ML */
{
  FldSetScrollPosition((FieldPtr)fptr, Int_val(pos));
  return Val_unit;
}

/* p237 : fld -> int -> int -> unit */
value clpalm_FldSetSelection(value fptr, value startpos, value endpos) /* ML */
{
  FldSetSelection((FieldPtr) fptr, Int_val(startpos), Int_val(endpos));
  return Val_unit;
}


value clpalm_FldSetText(value fptr, value handle, value offset, value size) /* ML */
{
  FldSetText((FieldPtr)fptr, (VoidHand)handle, Int_val(offset), Int_val(size));
  return Val_unit;
}

value clpalm_FldSetTextAllocatedSize(value fptr, value size) /* ML */
{
  FldSetTextAllocatedSize((FieldPtr)fptr, Int_val(size));
  return Val_unit;
}

value clpalm_FldSetTextHandle(value fptr, value handle) /* ML */
{
  FldSetTextHandle((FieldPtr)fptr, (Handle)handle);
  return Val_unit;
}


value clpalm_FldSetUsable(value fptr, value usable) /* ML */
{
  FldSetUsable((FieldPtr)fptr, Bool_val(usable));
  return Val_unit;
}

/* p241 : fld -> unit */
value clpalm_FldUndo(value fptr) /* ML */
{
  FldUndo((FieldPtr) fptr);
  return Val_unit;
}

value clpalm_FldWordWrap(value str, value max) /* ML */
{
  return Val_int(FldWordWrap(String_val(str), Int_val(max)));
}


/* 
 * Additional stuff
 */

/* fptr -> unit : this is required to avoid freeing when closing the form */
value clpalm_FldClearHandle(value fptr) /* ML */
{
  FldSetTextHandle((FieldPtr)fptr, (Handle)0);
  return Val_unit;
}

/* 
 * Fields used in connection with IO
 */

/* fptr -> in/out channel -> unit 
 * NOTE: the file on the channel MUST END with NULL character so
 * that the handle corresponds to a null terminated string.
 * This is normally the case for initial empty channels.
 */
value clpalm_FldSetChan(value fptr, struct channel *chan) /* ML */
{
  // FldSetText((FieldPtr)fptr, chan->record,chan->offset,0);
  FldSetTextHandle((FieldPtr)fptr, (Handle)chan->record);
  return Val_unit;
}


