/* Some of the Palm stuff */
#include "mlvalues.h"
#include "fail.h"
#include "alloc.h"
#include "str.h"
#include "io.h"
#include "clPalm_cvt.h"

/*
 * Form stuff
 */

/* p251 int -> int */
value clpalm_FrmAlert(value id) /* ML */
{
  return Val_int(FrmAlert(Int_val(id)));
}

/* FrmCloseAllForms is not normally needed because our main event loop calls it
   on exit */

/* p252 form -> id -> string -> unit 
   The doc says the string is copied (with size restrictions)
 */
value clpalm_FrmCopyLabel(value form, value id, value label) /* ML */
{
  FrmCopyLabel((FormPtr)form, Int_val(id), String_val(label));
  return Val_unit;
}

/* p253 form -> string -> unit 
   The doc says the string is copied (with size restrictions)
 */
value clpalm_FrmCopyTitle(value form, value title) /* ML */
{
  FrmCopyTitle((FormPtr)form, String_val(title));
  return Val_unit;
}

/* p254 : id -> string list -> int 
 * at most three strings in the list !
 */
value clpalm_FrmCustomAlert(value id, value params)  /* ML */
{
  CharPtr arg1 = NULL;
  CharPtr arg2 = NULL;
  CharPtr arg3 = NULL;
  if (Tag_val(params) == 1) {
    arg1 = (CharPtr)String_val(Field(params, 0));
    params = Field(params, 1);
    if (Tag_val(params) == 1) {
      arg2 = (CharPtr)String_val(Field(params, 0));
      params = Field(params, 1);
      if (Tag_val(params) == 1) {
	arg3 = (CharPtr)String_val(Field(params, 0));
      }
    }
  }
  return Val_int(FrmCustomAlert(Int_val(id), arg1, arg2, arg3));
}

/* p255: form -> unit */
value clpalm_FrmDeleteForm(value form) /* ML */
{
  FrmDeleteForm((FormPtr)form);
  return Val_unit;
}

value clpalm_FrmDispatchEvent(value ev) /* ML */
{
  EventType e;
  clpalm_mlevent2c(ev, &e);
  return Val_bool(FrmDispatchEvent(&e));
}

/* p256 form -> int */
value clpalm_FrmDoDialog(value form) /* ML */
{
  return Val_int(FrmDoDialog((FormPtr)form));
}

/* p256 form -> unit */
value clpalm_FrmDrawForm(value form) /* ML */
{
  FrmDrawForm((FormPtr)form);
  return Val_unit;
}

/* p257 form -> unit */
value clpalm_FrmEraseForm(value form) /* ML */
{
  FrmEraseForm((FormPtr)form);
  return Val_unit;
}

/* p257 unit -> form */
value clpalm_FrmGetActiveForm(value rien) /* ML */
{
  return (value)FrmGetActiveForm();
}


value clpalm_FrmGetActiveFormID(value rien) /* ML */
{
  return Val_int(FrmGetActiveFormID());
}

value clpalm_FrmGetControlGroupSelection(value form, value groupnum) /* ML */
{
  int g = FrmGetControlGroupSelection((FormPtr)form, Int_val(groupnum));
  if (g == -1)
    mlraise(NOT_FOUND_EXN);
  else
    return Val_int(g);
}

value clpalm_FrmGetControlValue(value form, value idx) /* ML */
{
  return Val_bool(FrmGetControlValue((FormPtr)form, Int_val(idx)));
}

value clpalm_FrmGetFirstForm(value rien) /* ML */
{
  FormPtr form = FrmGetFirstForm();
  return null_ptr((value)form);
}

value clpalm_FrmGetFocus(value form) /* ML */
{
  Word n = FrmGetFocus((FormPtr)form);
  if (n == -1) mlraise(NOT_FOUND_EXN);
  else return Val_int(n);
}

value clpalm_FrmGetFormBounds(value form) /* ML */
{
  RectangleType rect;
  FrmGetFormBounds((FormPtr)form, &rect);
  return rect_c2ml(&rect);
}

value clpalm_FrmGetFormId(value form) /* ML */
{
  return Val_int(FrmGetFormId((FormPtr)form));
}

value clpalm_FrmGetFormPtr(value id) /* ML */
{
  FormPtr f = FrmGetFormPtr(Int_val(id));
  return null_ptr((value)f);
}


value clpalm_FrmGetLabel(value fptr, value id) /* ML */
{
  return copy_string(FrmGetLabel((FormPtr)fptr, Int_val(id)));
}

value clpalm_FrmGetNumberOfObjects(value fptr) /* ML */
{
  return Val_int(FrmGetNumberOfObjects((FormPtr)fptr));
}

value clpalm_FrmGetObjectBounds(value fptr, value idx) /* ML */
{
  RectangleType rect;
  FrmGetObjectBounds((FormPtr)fptr, Int_val(idx), &rect);
  return rect_c2ml(&rect);
}

value clpalm_FrmGetObjectId(value fptr, value idx) /* ML */
{
  return Val_int(FrmGetObjectId((FormPtr)fptr, Int_val(idx)));
}

value clpalm_FrmGetObjectIndex(value fptr, value id) /* ML */
{
  return Val_int(FrmGetObjectIndex((FormPtr)fptr, Int_val(id)));
}

value clpalm_FrmGetObjectPosition(value fptr, value idx) /* ML */
{
  SWord x, y;
  value res;
  FrmGetObjectPosition((FormPtr)fptr, Int_val(idx), &x, &y);
  res = alloc_tuple(2);
  Field(res, 0) = Val_int(x);
  Field(res, 1) = Val_int(y);
  return res;
}

value clpalm_FrmGetTitle(value fptr) /* ML */
{
  return copy_string(FrmGetTitle((FormPtr)fptr));
}

value clpalm_FrmGetUserModifiedState(value fptr) /* ML */
{
  return Val_bool(FrmGetUserModifiedState((FormPtr)fptr));
}

value clpalm_FrmGetWindowHandle(value fptr) /* ML */
{
  return (value)FrmGetWindowHandle((FormPtr)fptr);
}

/* p268 int -> unit */
value clpalm_FrmGotoForm(value id) /* ML */
{
  FrmGotoForm(Int_val(id));
  return Val_unit;
}

/* p268 event -> bool */
value clpalm_FrmHandleEvent(value form, value ev) /* ML */
{
  EventType e;
  clpalm_mlevent2c(ev, &e);
  return Val_bool(FrmHandleEvent((FormPtr)form, &e));
}

value clpalm_FrmHelp(value id) /* ML */
{
  FrmHelp(Int_val(id));
  return Val_unit;
}

value clpalm_FrmHideObject(value fptr, value idx) /* ML */
{
  FrmHideObject((FormPtr)fptr, Int_val(idx));
  return Val_unit;
}

/* p270 int -> form */
value clpalm_FrmInitForm(value id) /* ML */
{
  return (value)FrmInitForm(Int_val(id));
}

value clpalm_FrmPointInTitle(value fptr, value x, value y ) /* ML */
{
  return Val_bool(FrmPointInTitle((FormPtr)fptr, Int_val(x), Int_val(y)));
}

value clpalm_FrmPopupForm(value id) /* ML */
{
  FrmPopupForm(Int_val(id));
  return Val_unit;
}

value clpalm_FrmReturnToForm(value id) /* ML */
{
  FrmReturnToForm(Int_val(id));
  return Val_unit;
}

/* p272 form -> unit */
value clpalm_FrmSetActiveForm(value form)	/* ML */
{
  FrmSetActiveForm((FormPtr)form);
  return Val_unit;
}

value clpalm_FrmSetControlGroupSelection(value fptr, value group, value id) /* ML */
{
  FrmSetControlGroupSelection((FormPtr)fptr, Int_val(group), Int_val(id));
  return Val_unit;
}

value clpalm_FrmSetControlValue(value fptr, value idx, value flag) /* ML */
{
  FrmSetControlValue((FormPtr)fptr, Int_val(idx), Bool_val(flag));
  return Val_unit;
}

/* p276 form -> int -> unit */
value clpalm_FrmSetFocus (value form, value idx) /* ML */
{
  FrmSetFocus((FormPtr)form, Int_val(idx));
  return Val_unit;
}

value clpalm_FrmSetNotUserModified(value fptr) /* ML */
{
  FrmSetNotUserModified((FormPtr)fptr);
  return Val_unit;
}

value clpalm_FrmSetMenu(value fptr, value id) /* ML */
{
  FrmSetMenu((FormPtr)fptr, Int_val(id));
  return Val_unit;
}

value clpalm_FrmSetObjectBounds(value fptr, value idx, value bounds) /* ML */
{
  RectangleType rect;
  rect_ml2c(bounds, &rect);
  FrmSetObjectBounds((FormPtr)fptr, Int_val(idx), &rect);
  return Val_unit;
}

value clpalm_FrmSetObjectPosition(value fptr, value idx, value x, value y) /* ML */
{
  FrmSetObjectPosition((FormPtr)fptr, Int_val(idx), Int_val(x), Int_val(y));
  return Val_unit;
}

value clpalm_FrmShowObject(value fptr, value idx) /* ML */
{
  FrmShowObject((FormPtr)fptr, Int_val(idx));
  return Val_unit;
}

value clpalm_FrmUpdateScrollers(value fptr, value upidx, value downidx, value scrup, value scrdown) /* ML */
{
  FrmUpdateScrollers((FormPtr)fptr, Int_val(upidx), Int_val(downidx), Bool_val(scrup), Bool_val(scrdown));
  return Val_unit;
}

value clpalm_FrmUpdateForm(value id, value redraw) /* ML */
{
  FrmUpdateForm(Int_val(id), Bool_val(redraw) ? frmRedrawUpdateCode : 0);
  return Val_unit;
}

value clpalm_FrmVisible(value fptr) /* ML */
{
  return Val_bool(FrmVisible((FormPtr)fptr));
}
