#include "mlvalues.h"
#include "fail.h"
#include "alloc.h"
#include "memory.h"
#include "str.h"
#include "sys.h"

#include "clPalm_cvt.h"

/*
 * Window functions
 */

value clpalm_WinAddWindow(value win) /* ML */
{
  WinAddWindow((WinHandle) win);
  return Val_unit;
}

value clpalm_WinClipRectangle(value vrect) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinClipRectangle(&rect);
  return Val_unit;
}

/* WARNING: 6 arguments, use C_CALLN 
 * CHECK: type op consistent with Window.h enum ScrOperation
 */
value clpalm_WinCopyRectangle(value *argv, value argn) /* ML */
     /* 0 value src
	1 value dst
	2 value vrect
	3 value destX
	4 value destY
	5 value mode */
{
  RectangleType rect;
  rect_ml2c(argv[2], &rect);
  WinCopyRectangle((WinHandle)argv[0], (WinHandle)argv[1], &rect,
		   Int_val(argv[3]), Int_val(argv[4]), Tag_val(argv[5])) ;
  return Val_unit;
}

/* FrameType as int, it's easier */
value clpalm_WinCreateWindow(value vrect, value ftype, value vmodal, value vfocusable) /* ML */
{
  Word error;
  RectangleType rect;
  WinHandle res;

  rect_ml2c(vrect, &rect);
  res = WinCreateWindow(&rect, Int_val(ftype), Bool_val(vmodal), Bool_val(vfocusable), &error);
  if (res == NULL) sys_error(error);
  return (value)res;
}

/* CHECK consistency with type window_format */
value clpalm_WinCreateOffscreenWindow(value w, value h, value format) /* ML */
{
  Word error;
  WinHandle res;

  res = WinCreateOffscreenWindow(Int_val(w), Int_val(h), Tag_val(format), &error);
  if (res == NULL) sys_error(error);
  return (value)res;
}

value clpalm_WinDeleteWindow(value win, value erase) /* ML */
{
  WinDeleteWindow((WinHandle)win, Bool_val(erase));
  return Val_unit;
}


value clpalm_WinDisableWindow(value win) /* ML */
{
  WinDisableWindow((WinHandle)win);
  return Val_unit;
}

value clpalm_WinDisplayToWindowPt(value extentX, value extentY) /* ML */
{
  value res;
  SWord x = Int_val(extentX);
  SWord y = Int_val(extentY);

  WinDisplayToWindowPt(&x, &y);
  res = alloc_tuple(2);		/* other param can move */
  Field(res, 0) = Val_int(x);
  Field(res, 1) = Val_int(y);
  return res;
}


value clpalm_WinDrawBitmap(value bptr, value x, value y) /* ML */
{
  WinDrawBitmap((BitmapPtr)bptr, Int_val(x), Int_val(y));
  return Val_unit;
}


/* p369 string -> int -> int -> int -> unit */
value clpalm_WinDrawChars(value chars, value len, value x, value y) /* ML */
{
  WinDrawChars(String_val(chars), Int_val(len), Int_val(x), Int_val(y));
  return Val_unit;
}

value clpalm_WinDrawGrayLine(value x1, value y1, value x2, value y2) /* ML */
{
  WinDrawGrayLine(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value clpalm_WinDrawGrayRectangleFrame(value ftype, value vrect) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinDrawGrayRectangleFrame(Int_val(ftype), &rect);
  return Val_unit;
}

value clpalm_WinDrawInvertedChars(value chars, value len, value x, value y) /* ML */
{
  WinDrawInvertedChars(String_val(chars), Int_val(len), Int_val(x), Int_val(y));
  return Val_unit;
}

value clpalm_WinDrawLine(value x1, value y1, value x2, value y2) /* ML */
{
  WinDrawLine(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value clpalm_WinDrawRectangle(value vrect, value corner) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinDrawRectangle(&rect, Int_val(corner));
  return Val_unit;
}

value clpalm_WinDrawRectangleFrame(value ftype, value vrect) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinDrawRectangleFrame(Int_val(ftype), &rect);
  return Val_unit;
}

value clpalm_WinDrawWindowFrame(value rien) /* ML */
{
  WinDrawWindowFrame();
  return Val_unit;
}

value clpalm_WinEnableWindow(value win) /* ML */
{
  WinEnableWindow((WinHandle)win);
  return Val_unit;
}

value clpalm_WinEraseChars(value chars, value len, value x, value y) /* ML */
{
  WinEraseChars(String_val(chars), Int_val(len), Int_val(x), Int_val(y));
  return Val_unit;
}

value clpalm_WinEraseLine(value x1, value y1, value x2, value y2) /* ML */
{
  WinEraseLine(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value clpalm_WinEraseRectangle(value vrect, value corner) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinEraseRectangle(&rect, Int_val(corner));
  return Val_unit;
}

value clpalm_WinEraseRectangleFrame(value ftype, value vrect) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinEraseRectangleFrame(Int_val(ftype), &rect);
  return Val_unit;
}

value clpalm_WinEraseWindow(value rien) /* ML */
{
  WinEraseWindow();
  return Val_unit;
}

value clpalm_WinFillLine(value x1, value y1, value x2, value y2) /* ML */
{
  WinFillLine(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value clpalm_WinFillRectangle(value vrect, value corner) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinFillRectangle(&rect, Int_val(corner));
  return Val_unit;
}


value clpalm_WinGetActiveWindow(value rien) /* ML */
{
  return (value)WinGetActiveWindow();
}

value clpalm_WinGetClip(value rien) /* ML */
{
  RectangleType rect;
  WinGetClip(&rect);
  return rect_c2ml(&rect);
}

value clpalm_WinGetDisplayExtent(value extentX, value extentY) /* ML */
{
  value res;
  SWord x = Int_val(extentX);
  SWord y = Int_val(extentY);

  WinGetDisplayExtent(&x, &y);
  res = alloc_tuple(2);		/* no other param can move */
  Field(res, 0) = Val_int(x);
  Field(res, 1) = Val_int(y);
  return res;
}

value clpalm_WinGetDisplayWindow(value rien) /* ML */
{
  return (value)WinGetDisplayWindow();
}

value clpalm_WinGetDrawWindow(value rien) /* ML */
{
  return (value)WinGetDrawWindow();
}

value clpalm_WinGetFirstWindow(value rien) /* ML */
{
  return (value)WinGetFirstWindow();
}

value clpalm_WinGetFramesRectangle(value ftype)/* ML */
{
  RectangleType rect, orect;
  WinGetFramesRectangle(Int_val(ftype), &rect, &orect);
  {
    value res;
    Push_roots(r, 2);
    r[0] = rect_c2ml(&rect);
    r[1] = rect_c2ml(&orect);
    res = alloc_tuple(2);
    Field(res, 0) = r[0];
    Field(res, 1) = r[1];
    Pop_roots();
    return res;
  }
}

value clpalm_WinGetPattern(value rien) /* ML */
{
  value res;
  CustomPatternType pat;
  WinGetPattern(pat);
  res = alloc_string(8);	/* no other param can move */
  bcopy(pat, String_val(res), 8);
  return res;
}

value clpalm_WinGetWindowBounds(value rien) /* ML */
{
  RectangleType rect;
  WinGetWindowBounds(&rect);
  return rect_c2ml(&rect);
}

value clpalm_WinGetWindowExtent(value extentX, value extentY) /* ML */
{
  value res;
  SWord x = Int_val(extentX);
  SWord y = Int_val(extentY);

  WinGetWindowExtent(&x, &y);
  res = alloc_tuple(2);
  Field(res, 0) = Val_int(x);
  Field(res, 1) = Val_int(y);
  return res;
}

value clpalm_WinGetWindowFrameRect(value win) /* ML */
{
  RectangleType rect;
  WinGetWindowFrameRect((WinHandle)win, &rect);
  return rect_c2ml(&rect);
}

/* WinGetWindowPointer */

value clpalm_WinInitializeWindow(value win) /* ML */
{
  WinInitializeWindow((WinHandle)win);
  return Val_unit;
}

value clpalm_WinInvertChars(value chars, value len, value x, value y) /* ML */
{
  WinInvertChars(String_val(chars), Int_val(len), Int_val(x), Int_val(y));
  return Val_unit;
}

value clpalm_WinInvertLine(value x1, value y1, value x2, value y2) /* ML */
{
  WinInvertLine(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value clpalm_WinInvertRectangle(value vrect, value corner) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinInvertRectangle(&rect, Int_val(corner));
  return Val_unit;
}

value clpalm_WinInvertRectangleFrame(value ftype, value vrect) /* ML */
{
  RectangleType rect;
  rect_ml2c(vrect, &rect);
  WinInvertRectangleFrame(Int_val(ftype), &rect);
  return Val_unit;
}

value clpalm_WinModal(value win) /* ML */
{
  return Val_bool(WinModal((WinHandle)win));
}

value clpalm_WinRemoveWindow(value win) /* ML */
{
  WinRemoveWindow((WinHandle)win);
  return Val_unit;
}

value clpalm_WinResetClip(value rien) /* ML */
{
  WinResetClip();
  return Val_unit;
}

value clpalm_WinRestoreBits(value win, value x, value y) /* ML */
{
  WinRestoreBits((WinHandle)win, Int_val(x), Int_val(y));
  return Val_unit;
}

value clpalm_WinSaveBits(value vrect) /* ML */
{
  RectangleType rect;
  WinHandle win;
  Word error;

  rect_ml2c(vrect, &rect);
  win = WinSaveBits(&rect, &error);
  if (win == NULL) sys_error(error);
  return (value)win;
}

value clpalm_WinScrollRectangle(value vrect, value dir, value dist) /* ML */
{
  RectangleType rect, vacated;

  rect_ml2c(vrect, &rect);
  WinScrollRectangle(&rect, Tag_val(dir), Int_val(dist), &vacated);
  return rect_c2ml(&vacated);
}

value clpalm_WinSetActiveWindow(value win) /* ML */
{
  WinSetActiveWindow((WinHandle)win);
  return Val_unit;
}

value clpalm_WinSetClip(value vrect) /* ML */
{
  RectangleType rect;

  rect_ml2c(vrect, &rect);
  WinSetClip(&rect);
  return Val_unit;
}

value clpalm_WinSetDrawWindow(value win) /* ML */
{
  return (value)WinSetDrawWindow((WinHandle)win);
}

value clpalm_WinSetPattern(value vpat) /* ML */
{
  WinSetPattern((Word *)String_val(vpat));
  return Val_unit;
}

value clpalm_WinSetUnderlineMode(value umode) /* ML */
{
  return Atom(WinSetUnderlineMode(Tag_val(umode)));
}

value clpalm_WinSetWindowBounds(value win, value vrect) /* ML */
{
  RectangleType rect;
  
  rect_ml2c(vrect, &rect);
  WinSetWindowBounds((WinHandle)win, &rect);
  return Val_unit;
}

value clpalm_WinWindowToDisplayPt(value extentX, value extentY) /* ML */
{
  value res;
  SWord x = Int_val(extentX);
  SWord y = Int_val(extentY);

  WinWindowToDisplayPt(&x, &y);
  res = alloc_tuple(2);
  Field(res, 0) = Val_int(x);
  Field(res, 1) = Val_int(y);
  return res;
}
