#include "mlvalues.h"
#include "fail.h"

extern value current_exn();	/* get the current exception */
extern void set_external_raise(struct longjmp_buffer *b);
extern Boolean ApplicationHandleEvent(EventPtr event);

/* name of exceptions 
 * TODO: we might as well use the generic error msg of PalmOS here.
 * II/114
 */
char *uncaught(void)
{
  static char buf[] = "        ";
  char *name;
  value exn = current_exn();

  switch (Tag_val(exn)) {
  case OUT_OF_MEMORY_EXN:
    name = "Out of memory";
    break;
  case SYS_ERROR_EXN:
  case FAILURE_EXN:
  case INVALID_EXN:
    name = String_val(Field(exn,0));
    break;
  case END_OF_FILE_EXN:
    name = "End_of_file";
    break;
  case ZERO_DIVIDE_EXN:
    name = "Division_by_zero";
    break;
  case BREAK_EXN:
    name = "Break";
    break;
  case NOT_FOUND_EXN:
    name = "Not_found";
    break;
  default:
    sprintf(buf,"Tag: %d", Tag_val(exn));
    name = buf;
    break;
  };
  return name;
}

/* This is a standard well-behaved mainloop */
void EventLoop(void)
{
  EventType event;
  Err err;
  do {
    EvtGetEvent(&event, -1); /* wait forever */
    if (SysHandleEvent(&event)) continue; /* the system handles it */
    if (MenuHandleEvent(0, &event, &err)) continue; /* the menu handles it */
    if (ApplicationHandleEvent(&event)) continue;
    FrmDispatchEvent(&event);
  } while (event.eType != appStopEvent);
}

/*
 * In general, we must call FrmCloseAllForms, which it turn will invoke
 * each form handler with a frmCloseEvent. These handler should reset the
 * field text handles, at least.
 * Currently FrmCloseAllForms is hardcoded here, but it should move
 * to at_exit in Caml.
 */
void StopApplication(int fatal)
{
  struct longjmp_buffer raise_buf;

  if (fatal) {
    /* we already longjmp-ed to the error exit handler because of an uncaught
     * exception in some callback. Thus we setjmp again and simulate
     * an appStopEvent (essentially: we want a clean exit).
     */
    if (setjmp(&raise_buf.buf[0]) == 0) {
      EventType e;
      /* Where to go if we die  */
      set_external_raise(&raise_buf);
      /* Invoke appStopEvent */
      e.eType = appStopEvent;
      ApplicationHandleEvent(&e); /* let the app do some cleanup */
      FrmCloseAllForms(); /* this should move to at_exit */
    } else { /* damn. yet another exception. */
      FrmCustomAlert(2, uncaught(),NULL,NULL);
    }      
  } else {
    /* normal exit : this calls some Caml code (frmCloseEvent) */
    /* at_exit handler was already invoked by ApplicationHandleEvent
     * before we arrive here. We are still protected by the main setjmp.
     */
    FrmCloseAllForms(); /* this should move to at_exit */
  }
}

