#include "config.h"
#include "globals.h"
#include "mlvalues.h"
#include "memory.h"
#include "gc.h"
#include "gc_ctrl.h"
#include "stacks.h"
#include "fail.h"
#include "prims.h"
#include "dispatch.h"

/* These are not used on PalmOS */
Volatile code_t pending_signal_handler;
Volatile int pending_signal = 0;

/* These are globals that we want to access from
   the other parts of the Caml application
 */

/* for dialog box on uncaught exceptions */
value current_exn ()
{
  return exn_bucket;
}

/* needed mostly for the initial load of globals */
void set_external_raise(struct longjmp_buffer *b)
{
  external_raise = b;
}

void sys_init(value globs)
{
  global_data = globs;
  modify(&Field(global_data, GLOBAL_DATA), global_data);
  Field(global_data, SYS__MAX_VECT_LENGTH) = Val_long(Max_wosize);
  Field(global_data, SYS__MAX_STRING_LENGTH) =
    Val_long(Bsize_wsize (Max_wosize) - 1);
#ifdef CAML_SIXTYFOUR
  Field(global_data, SYS__WORD_SIZE) = Val_long (64);
#else
  Field(global_data, SYS__WORD_SIZE) = Val_long (32);
#endif
}

/* Atoms */
header_t *first_atoms = NULL;

header_t *get_atoms()
{
  if (first_atoms == NULL) {
    VoidHand atomsh;
    DmOpenRef dm;
    dm = DmOpenDatabaseByTypeCreator('GLib', 'CLRT', dmModeReadOnly);
    ErrFatalDisplayIf(dm == NULL, "CLRT"); 
    atomsh = DmGet1Resource('CLat', 0);
    ErrFatalDisplayIf(atomsh == NULL, "CLat CLRT"); 
    first_atoms = MemHandleLock(atomsh);
  }
  return first_atoms;
}

void free_atoms()
{
  if (first_atoms) {
    VoidHand atomsh;
    DmOpenRef dm;

    atomsh = MemPtrRecoverHandle(first_atoms);
    ErrFatalDisplayIf(atomsh == NULL, "atoms");
    MemHandleUnlock(atomsh);
    DmSearchResource('CLat', 0, atomsh, &dm);
    ErrFatalDisplayIf(dm == NULL, "atoms rsrc");
    DmReleaseResource(atomsh);
    DmCloseDatabase(dm);
  }
}

#define MAXLAYERS 3
c_primitive *primsa[MAXLAYERS];	 /* table of primitives arrays */
int numprims[MAXLAYERS];	 /* sizes of primitives arrays */
void *globalp[MAXLAYERS];	 /* a4 pointers for layers */

void declare_prims(int layer, c_primitive *p, int len, void *layer_a4)
{
  primsa[layer] = p;
  globalp[layer] = layer_a4;
  numprims[layer] = len;
}

extern c_primitive cprims1[];
extern int cprims1n;

void runtime_init(int verb)
{
  init_gc (Minor_heap_def, Heap_chunk_def, Percent_free_def, verb);
  init_stacks();
  declare_prims(0, cprims1, cprims1n, reg_a4);  /* declare our own prims  */
}

