/* Buffered input/output simulation */

#ifndef _io_
#define _io_

#include "misc.h"
#include "mlvalues.h"

struct channel {
  VoidHand record;		/* the record */
  /* The record is kept locked while it is made available in Caml.
   * It is the responsability of the application to close all opened channels,
   * otherwise the busy bit might stay and cause an error on the next invocation
   * of the application (this until we get DmResetRecordStates to work !)
   */
  ULong offset;			/* position in the file */
  ULong max;			/* max size */
  /* For input channels : 
   *    max is initialized to the size of the record. This size is only meaningful
   *    (in terms of what is actually available on the channel) if the record is
   *    fully used. However, this relies on application invariants, such as the
   *    use of fields or io functions.
   *    Fields: hypothesis is that real data is defined by null-termination
   *    UserIO: asssume close_out resizes the record to its exact size.
   *  The application needs to call sync_field in case it uses a field
   *  to edit the record.
   * For output channels :
   *    max means the actual used size of the record.
   */
  UInt index;			/* the record index */
  Boolean resource : 1;		/* is a resource */
  /* this bit is internal : it tells us how to close/release the record */
  Boolean binary : 1;		/* binary mode : can't use in fields */
  /* for input "text" channels, that application can use if fields,
   * max is one less than the actual size of the record at init-time.
   * because we assume that the text is NULL-terminated 
   */
};

#define DEFAULT_SIZE 512

uint32 getword P((struct channel *));
unsigned getblock P((struct channel *, char *, unsigned));

void putword P((struct channel *, uint32));
void putblock P((struct channel *, char *, unsigned));

#define really_getblock(c,p,l)     getblock(c,p,l)

extern value close_in(struct channel *chan);

#endif /* _io_ */
