#include "config.h"
#include "mlvalues.h"
#include "memory.h"
#include "fail.h"
#include "sys.h"
#include "alloc.h"
#include "io.h"

#include "clPalm_cvt.h"

static DmOpenRef dm = 0;

/* This has to be called at "startup" : returns true if all OK
 * It makes sure the default "filesystem" is setup.
 */
value clpalmio_init(value vtype, value vcreator, value vname) /* ML */
{
  ULong type = id_of_string(vtype);
  ULong creator = id_of_string(vcreator);
  CharPtr name = String_val(vname);

  dm = DmOpenDatabaseByTypeCreator(type, creator, dmModeReadWrite);
  if (dm == 0) {
    Err err;
    err = DmCreateDatabase(0, name, creator, type, false);
    if (err == 0) {
      dm = DmOpenDatabaseByTypeCreator(type, creator, dmModeReadWrite);
      /* we assume this can't fail since creation succeeded */
      return Val_unit;
    }
    else {
      ErrFatalDisplayIf(err,"Could not open/create database");
      sys_error(err);
    }
  }
  return Val_unit;
}

value clpalmio_exit(value rien)	/* ML */
{
  if (dm) {
    /* This always causes a bus-error on xcopilot in my tests. 
     * DmResetRecordStates(dm); // make sure all records are unlocked
     */
    DmCloseDatabase(dm);
    dm = 0;
  }
  return Val_unit;
}

/* open an existing record from the standard DB. */
struct channel * open_record(int index, int binary)
{
  struct channel *chan;

  chan = (struct channel *) stat_alloc (sizeof(struct channel));
  chan->index = index;

  /* use DmQueryRecord first, because DmGetRecord makes fatal errors
   * when index is out of range. Anyway, DmGetRecord makes a fatal error
   * if record has busy bit set.
   */
  chan->record = DmQueryRecord(dm, chan->index);
  if (chan->record == 0) {
    stat_free((void *)chan);
    sys_error(DmGetLastErr());
  }
  else {
    chan->record = DmGetRecord(dm, chan->index);
    chan->offset = 0L;
    chan->resource = false;
    chan->binary = binary;
    chan->max = MemHandleSize(chan->record);
    /* TODO: check it's > 0 ! */
    if (!binary) chan->max = chan->max - 1;
    return chan;
  }
}

value open_atzero(value name)	/* ML */
{
  return (value)open_record(Int_val(name), false);
}

value open_atend(value name)	/* ML */
{
  struct channel *chan = open_record(Int_val(name), false);
  chan->offset = chan->max;
  return (value)chan;
}

/* open/truncate if it exists, or create if not, a "text-mode" record */
/* The record is created with a default size; a field may write in it */
value open_create(value name)	/* ML */
{
  struct channel *chan;

  chan = (struct channel *) stat_alloc (sizeof(struct channel));
  chan->index = Int_val(name);
  chan->offset = 0L;
  chan->max = 0L; /* ok when create, means trunc when open */
  chan->resource = false;
  chan->binary = false;
	    
  chan->record = DmQueryRecord(dm, chan->index);
  if (chan->record == 0) {	/*  doesn't exist yet */
    /* can't create a record with size 0... error 0201 */
    chan->record = DmNewRecord(dm, &chan->index, DEFAULT_SIZE);
  } else { /* it's there */
    chan->record = DmGetRecord(dm, chan->index);
  };
  
  if (chan->record == 0) {
    stat_free((char *)chan);
    sys_error(DmGetLastErr());
  } else { /* make the first character a zero, since this is text+trunc */
    Char zero = 0;
    VoidPtr recordP;

    recordP = MemHandleLock(chan->record);
    DmWrite(recordP, 0, &zero, sizeof(Char));
    MemHandleUnlock(chan->record);
    return (value)chan;
  }
}

/* Reset the max value of an input_channel according to a given field
 * (e.g. this record is actually edited in this field
 */
value sync_field(struct channel *chan, FieldPtr fld)	/* ML */
{
  chan->max = FldGetTextLength(fld);
  return Val_unit;
}

value print_debug(value str)	/* ML */
{
  printf("%s\n", String_val(str));
  return Val_unit;
}

#include <ConsoleMgr.h>
value con_debug(value str)	/* ML */
{
  ConPutS(String_val(str));
  return Val_unit;
}


