#include "config.h"
#include "fail.h"

/* PalmOS 2.0 Support 
 * Requires prc-tools-0.5

 * A simple solution is 
 #define malloc(x) MemPtrNew(x)
 #define free(x) MemPtrFree(x)
 with realloc as MemPtrResize (+ MemPtrNew/MemCpy/MemPtrFree if resize failed)
 
 The other solution is to use moveable chunks, but always locked (except
 when we want to realloc).
*/

UInt owner = 0;
void init_alloc (UInt o)
{
  owner = o;
}

void *malloc(size_t size)
{
  VoidHand h;
  char *ptr;
  h = MemHandleNew(size);
  if (h == NULL) {
    // printf("malloc %ld failed\n", size);
    return NULL;
  };
  MemHandleSetOwner(h, owner);
  ptr = MemHandleLock(h);
  // printf("malloc %ld = %ld\n", size, ptr);
  return ptr;
}

void free (void *ptr)
{
  VoidHand h = MemPtrRecoverHandle(ptr);
  if (h != NULL)
    MemHandleFree(h);		/* don't need to unlock */
}

void *realloc(void *ptr, size_t size)
{
  VoidHand h = MemPtrRecoverHandle(ptr);
  if (h == NULL) return NULL;
  if (MemHandleUnlock(h) == 0) {
    if (MemHandleResize (h, size) == 0) {
      MemHandleSetOwner(h, owner);
      return MemHandleLock(h);
    }
    else
      return NULL;
  }
  else /* failed to unlock ! */
    return NULL;
}

/* Convert a string to an int. strtol belongs to GCC library */
extern long int strtol(const char *nptr, char **endptr, int base);

int
atoi (const char *nptr)
{
  return (int) strtol (nptr, (char **) NULL, 10);
}

/*
 * Debugging support
 */
void ui_gc_message(char *msg, unsigned long arg)
{
  char buffer[128];		/* hope it's enough */
  sprintf(buffer,msg,arg);
  FrmCustomAlert(1,buffer,NULL,NULL);
}

void ui_fatal_error(char *fmt, char *arg)
{
  ErrFatalDisplayIf(1, arg);
}

struct longjmp_buffer *exit_buf;
void ui_set_exit(struct longjmp_buffer *b)
{
  exit_buf = b;
}

void ui_exit(int ret)
{
  longjmp(exit_buf->buf, ret);
}

void debug_int(long n)
{
  static char buf[16];
  sprintf(buf, "0x%8lx", n);
  FrmCustomAlert(0, buf, NULL, NULL);
}
