/* PalmOS : give a .zo, extract the bytecode and the globals */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "exec.h"

/* This is a cross-compiled program : do NOT use config.h */
typedef unsigned char *code_t;
typedef size_t asize_t;

static unsigned long read_size(p)
     unsigned char * p;
{
  return ((unsigned long) p[0] << 24) + ((unsigned long) p[1] << 16) +
         ((unsigned long) p[2] << 8) + p[3];
}

#define FILE_NOT_FOUND (-1)
#define TRUNCATED_FILE (-2)
#define BAD_MAGIC_NUM (-3)

static int read_trailer(fd, trail)
     int fd;
     struct exec_trailer * trail;
{
  char buffer[TRAILER_SIZE];

  lseek(fd, (long) -TRAILER_SIZE, 2);
  if (read(fd, buffer, TRAILER_SIZE) < TRAILER_SIZE) return TRUNCATED_FILE;
  trail->code_size = read_size(buffer);
  trail->data_size = read_size(buffer+4);
  trail->symbol_size = read_size(buffer+8);
  trail->debug_size = read_size(buffer+12);
  trail->magic = read_size(buffer+16);
  if (trail->magic == EXEC_MAGIC) return 0; else return BAD_MAGIC_NUM;
}

char * searchpath(name)
     char * name;
{
  static char fullname[512];
  char * path;
  char * p;
  char * q;
  struct stat st;

  for (p = name; *p != 0; p++) {
    if (*p == '/') return name;
  }
  path = getenv("PATH");
  if (path == 0) return 0;
  while(1) {
    p = fullname;
    while (*path != 0 && *path != ':') {
      *p++ = *path++;
    }
    if (p != fullname) *p++ = '/';
    q = name;
    while (*q != 0) {
      *p++ = *q++;
    }
    *p = 0;
    if (access(fullname, 1) == 0 &&
        stat(fullname, &st) == 0 &&
        (st.st_mode & S_IFMT) == S_IFREG)
      return fullname;
    if (*path == 0) return 0;
    path++;
  }
}

int attempt_open(name, trail, do_open_script)
     char ** name;
     struct exec_trailer * trail;
     int do_open_script;
{
  char * truename;
  int fd;
  int err, n;
  char buf [2];

  truename = searchpath(*name);
  if (truename == 0) truename = *name; else *name = truename;
  fd = open(truename, O_RDONLY);
  if (fd == -1) return FILE_NOT_FOUND;
  if (!do_open_script){
    n = read (fd, buf, 2);
    if (n < 2) { close(fd); return TRUNCATED_FILE; }
    if (buf [0] == '#' && buf [1] == '!') { close(fd); return BAD_MAGIC_NUM; }
  }
  err = read_trailer(fd, trail);
  if (err != 0) { close(fd); return err; }
  return fd;
}

/* Switch immediate operands from little endian to big endian. */
#ifdef 0
#include "instruct.h"
#include "reverse.h"

void fixup_endianness(p, len)
     register code_t p;
     asize_t len;
{
  register code_t q;
  int n;

  q = p + len;
  while(p < q) {
    switch(*p++) {
      /* Instructions with one one-byte + one two-byte argument */
    case BRANCHIFNEQTAG: case C_CALLN:
      p++; /* fall through */
      /* Instructions with a two-byte immediate argument */
    case GETGLOBAL: case SETGLOBAL: 
    case PUSH_GETGLOBAL_APPLY: case PUSH_GETGLOBAL_APPTERM:
    case CONSTSHORT:
    case CUR: case LETREC1: case PUSHTRAP:
    case BRANCH: case BRANCHIF: case BRANCHIFNOT: case POPBRANCHIFNOT:
    case BRANCHIFEQ: case BRANCHIFNEQ: case BRANCHIFLT:
    case BRANCHIFGT: case BRANCHIFLE: case BRANCHIFGE:
    case C_CALL1: case C_CALL2: case C_CALL3: case C_CALL4:
    case C_CALL5:
      Reverse_short(p);
      p += 2;
      break;
      /* Instructions with a four-byte immediate argument */
    case MAKEBLOCK:
      Reverse_word(p);
      p += 4;
      break;
      /* Instructions with two two-byte immediate arguments */
    case BRANCHINTERVAL:
      Reverse_short(p);
      Reverse_short(p+2);
      p += 4;
      break;
      /* Special case for switch */
    case SWITCH:
      n = *p++;
      while (n > 0) {
        Reverse_short(p);
        p += 2;
        n--;
      }
      break;
      /* Instructions with a one-byte immediate argument */
    case CONSTBYTE: case MAKEBLOCK1: case MAKEBLOCK2: case MAKEBLOCK3:
    case MAKEBLOCK4: case ACCESS: case ATOM: case GETFIELD: case SETFIELD:
    case DUMMY: case ENDLET: case FLOATOP:
      p++;
      break;
    }
  }
}

#endif

#define CLby "CLby0000.bin"
#define CLgl "CLgl0000.bin"

int main(int argc, char *argv[])
{
  int fd;
  struct exec_trailer trail;
  code_t start_code;
  asize_t code_size;
  char *start_data;

  fd = attempt_open(&argv[1], &trail, 0);

  lseek(fd, - (long) (TRAILER_SIZE + trail.code_size + trail.data_size
		      + trail.symbol_size + trail.debug_size), 2);

  code_size = trail.code_size;
  start_code = (code_t) malloc(code_size);
  if (read(fd, (char *) start_code, code_size) != code_size) {
    fprintf(stderr, "Fatal error: truncated bytecode file.\n");
    exit(1);
  }
  start_data = (char *)malloc(trail.data_size);
  if (read(fd, (char *) start_data, trail.data_size) != trail.data_size) {
    fprintf(stderr, "Fatal error: truncated bytecode file.\n");
    exit(1);
  }
  {
    int ofd;
    /* Write the bytecode file */
    ofd = open(CLby, O_CREAT|O_WRONLY|O_TRUNC, 0644);
    if (ofd == -1) {
      perror("Can't open bytecode file " CLby);
      exit(1);
    };
    write(ofd,start_code,code_size);
    close(ofd);

    /* The global data */
    ofd = open(CLgl, O_CREAT|O_WRONLY|O_TRUNC,0644);
    if (ofd == -1) {
      perror("Can't open global data file " CLgl);
      exit(1);
    }
    write(ofd,start_data,trail.data_size);
    close(ofd);
  }
  exit(0);
}
