/*
    Jump - Java post-compiler for PalmPilot
    Copyright (C) 1996,97  Greg Hewgill <gregh@lightspeed.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
    This file has been modified in 03-05/2000 
    by Ralf Kleberhoff <Kleberhoff@aol.com>
    to make Jump compatible with current Java and PalmOS versions
    and to extend the functionality.
    Many thanks to Greg Hewgill for the original version!
*/

import java.io.*;
import java.util.*;


class Jump implements JVM {

  static int CHARSIZE = 1; // or 2

  static final boolean ARRAYPADDING = true;

  static String Nativepath;
  static String Javac;
  static String Pila;

  /** 
   * flag: while resolving dependencies, we detected 
   * new 'needed' elements, meaning that we have to
   * go into one more iteration to find needed elements.
   */
  static boolean newNeeds;

  /**
   * a CodeOptions instance controlling the code generation.
   */
  static CodeOptions codeOptions = new CodeOptions();

  static boolean CompileCheck = false;
  static boolean DeleteAsm = false;
  static boolean Listing = false;
  static boolean Optimize = false;
  static boolean DebugSymbols = false;
  static int Verbosity = 1;
  static boolean Breakpoint = false;

  // static boolean NativeMethodsMissing = false;

  // static Hashtable Native = new Hashtable();

  static Properties projectProps;
  static String mainClassName = null;
  static BackEnd backEnd = null;

  static void ProcessOptions(String opt)
  {
    for (int i = 0; i < opt.length(); i++) {
      switch (Character.toLowerCase(opt.charAt(i))) {
      case 'c':
	CompileCheck = true;
	break;
      case 'd':
	DeleteAsm = true;
	break;
      case 'l':
	Listing = true;
	break;
      case 'o':
	Optimize = true;
	break;
      case 's':
	DebugSymbols = true;
	break;
      case 'v':
	Verbosity = 2;
	break;
	// rk: new option '-b': Debuffer breakpoint in startup code
      case 'b':
	Breakpoint = true;
	break;
      case '?':
	System.out.print(Usage);
	System.exit(1);
      }
    }
  }

  static int Run(String cmdline) throws IOException
  {
    System.out.println("Running: " + cmdline);
    Process p = Runtime.getRuntime().exec(cmdline);
    InputStream in = p.getInputStream();
    while (true) {
      int c = in.read();
      if (c < 0) {
        break;
      }
      System.out.write(c);
    }
    return p.exitValue();
  }

  static final String Usage =
    "Usage: java Jump [-cdlosv] classname\n\n"+
    "Options: -v  Verbose output\n"+
    "         -b  Insert breakpoint (!!! only for POSE + Debuffer !!!)\n";

  public static void main(String[] args)
  {
    System.out.println(
      "Jump  Java(tm) User Module for Pilot  Version 2.0 Alpha 8\n"+
      "Copyright (c) 1996,1997 by Greg Hewgill\n"+
      "Copyright (c) 2000 by Ralf Kleberhoff <kleberhoff@aol.com>\n"+
      "Please see the file COPYING (GPL v2) for distribution rights.\n");
    try {
      InputStream propStream;

      projectProps = new Properties ();

      propStream = ClassLoader.getSystemResourceAsStream ("Jump.properties");
      if (propStream != null) {
        projectProps.load (propStream);
      }

      for (int i = 0; i < args.length; i++) {
        if (args[i].charAt(0) == '-') {
        } else if (mainClassName == null) {
          mainClassName = args[i];
        } else {
          System.out.println("Only specify one class name on the command line.");
          System.exit(1);
        }
      }

      if (mainClassName == null) {
        System.out.print(Usage);
        System.exit(1);
      }
      if (mainClassName.toLowerCase().endsWith(".class")) {
        mainClassName = mainClassName.substring(0, mainClassName.length()-6);
      }

      propStream = ClassLoader.getSystemResourceAsStream (mainClassName + ".jump");
      if (propStream != null) {
        projectProps.load (propStream);
      }

      // Javac = new String(projectProps.getProperty("javac", "javac"));
      // Pila = new String(projectProps.getProperty("pila", "pila"));
      
      ProcessOptions(projectProps.getProperty("options", ""));
      for (int i = 0; i < args.length; i++) {
        if (args[i].charAt(0) == '-') {
          ProcessOptions(args[i]);
        }
      }

      System.out.println("Compiling: " + mainClassName + ".class");

      backEnd = new AsmBackEnd (mainClassName, null);

      Klass mainclass = Klass.forName(mainClassName);
      for (int i=0; i<mainclass.methods.length; i++) {
	mainclass.methods[i].markNeeded("method-of-main-class",JavaElement.NEEDED);
      }

      Klass.forName("java/lang/String").
	findMethod("<init>","([C)V").markNeeded("java-platform",JavaElement.NEEDED);

      /* I don't think they are needed for PalmOS.
      Klass.forName ("java/io/PrintStream").
	markNeeded("java-platform",JavaElement.INSTANCE_NEEDED);
      Klass.forName ("java/io/InputStream").
	markNeeded("java-platform",JavaElement.INSTANCE_NEEDED);
      Klass.forName ("java/lang/Class").
	markNeeded("java-platform",JavaElement.INSTANCE_NEEDED);
      */

      /* TODO: should find a better way to set the starting point(s) of an application
      MethodInfo mainMethod;
      if (mainMethod = mainclassfile.findMethod("PilotMain", "(III)I")) {
	mainMethod.markNeeded ("PilotMain",JavaElement.NEEDED);
      }
      if (mainMethod = mainclassfile.findMethod("main", "([Ljava/lang/String;)V")) {
	mainMethod.markNeeded ("java-main",JavaElement.NEEDED);
      }
      Klass.forName ("palmos/Event").markNeeded("PalmOS",JavaElement.INSTANCE_NEEDED);
      */
      
      do {
	newNeeds = false;
	Klass.updateAll();
	backEnd.updateAll();
      } while (newNeeds);

      Klass.assignAllClassIndices();
      Klass.layoutFieldsAll();
      Klass.layoutVtableAll();
      Klass.layoutItableAll();

      Klass.reportAll();

      JumpStatistics stat = new JumpStatistics();
      for (int i=0; i<Klass.ClassList.size(); i++) {
	Klass cl = (Klass) Klass.ClassList.elementAt(i);
	stat.addClass (cl);
	if (cl.methods != null) {
	  for (int j=0; j<cl.methods.length; j++) {
	    stat.addMethod (cl.methods[j]);
	  }
	}
      }
      stat.report();

      // Klass.writeNeededAll ("e:\temp");

      // generate target output here
      backEnd.generate();
    } 
    catch (Throwable e) {
      e.printStackTrace();
      System.exit(1);
    }
  }
}
