/*
    Jump - Java post-compiler for PalmPilot
    Copyright (C) 1996,97  Greg Hewgill <gregh@lightspeed.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
    This file has been modified in 03/2000 
    by Ralf Kleberhoff <Kleberhoff@aol.com>
    to make Jump compatible with current Java and PalmOS versions
    and to extend the functionality.
    Many thanks to Greg Hewgill for the original version!
*/

import java.io.*;
import java.util.*;

class JumpStatistics implements JVM {
  int nClasses = 0;
  int nInterfaces = 0;
  int nJavaMethods = 0;
  int nNativeMethods = 0;
  int nVirtualMethods = 0;
  int nInterfaceMethods = 0;
  int nBytesBytecode = 0;
  
  /**
   * create a JumpStatistics.
   */
  JumpStatistics ()
  {
  }

  void addMethod (MethodInfo method)
  {
    Klass cl = method.cls;

    if (method.isNeeded(JavaElement.NEEDED)) {
      if (((method.access_flags & ACC_NATIVE) != 0) ||
	  (method.nativeRef != null)) {
	nNativeMethods++;
      }
      else {
	nJavaMethods++;
	nBytesBytecode += method.attributes.code.code.length;
      }
    }

    if (method.isNeeded(JavaElement.NEEDED_VIRTUALLY)) {
      if ((cl.access_flags & ACC_INTERFACE) != 0) {
	nInterfaceMethods++;
      }
      else {
	nVirtualMethods++;
      }
    }
  }

  void addClass (Klass cls)
  {
    if (cls.isNeeded(JavaElement.NEEDED)) {
      if ((cls.access_flags & ACC_INTERFACE) != 0) {
	nInterfaces++;
      }
      else {
	nClasses++;
      }
    }
  }

  /**
   * print the statistics to System.out.
   */
  void report()
  {
    System.out.println ("         nClasses = " +          nClasses);
    System.out.println ("      nInterfaces = " +       nInterfaces);
    System.out.println ("     nJavaMethods = " +      nJavaMethods);
    System.out.println ("   nNativeMethods = " +    nNativeMethods);
    System.out.println ("  nVirtualMethods = " +   nVirtualMethods);
    System.out.println ("nInterfaceMethods = " + nInterfaceMethods);
    System.out.println ("    bytecode size = " +    nBytesBytecode);
  }

}
