;; kernel-routines

;; 
;; Kernel Routines for Jump2
;; always included into asm output
;;

;;; returns a pointer to the ClassInfo structure
;;; for an object - Object.getClass()
;;; IN  a0 = &object
;;; OUT a0 = &ClassInfo
getclassinfo_a0:
	move.w	ObjectHeader_ClassIndex(a0),d0
getclassinfo:
	mulu.w	#ClassInfo_size,d0
	lea	ClassTable(pc),a0
	add.l	d0,a0
	rts

;; does object (a0) belong to the class #d0.w ?
;; result: d0.l = 1 / 0
;; TODO: similar routine for interfaces
instancecheck:
	move.w	d0,d1
	move.w	ObjectHeader_ClassIndex(a0),d0
instancecheck_loop:
	cmp.w	d0,d1
	beq.s	instancecheck_ok
	bsr.s	getclassinfo
	move.w	ClassInfo.SuperClass(a0),d0
	bpl.s	instancecheck_loop
	clr.l	d0
	rts
instancecheck_ok:
	clr.l	d0
	addq.l	#1,d0
	rts

;; uses-class java/lang/String as String_class
makevoidptr:
	move.l  4(a7),a0
	cmp.w	#0,a0
	beq.s	makevoidptr_out
	cmp.w	#String_class,ObjectHeader_ClassIndex(a0)
	beq.s	makevoidptr_string
	bsr.s   getclassinfo_a0
	move.w  ClassInfo.Flags(a0),d0
	and.w   #ClassInfo_array,d0
	beq.s   makevoidptr_out
	move.l  4(a7),a0
	move.l  Array.data(a0),4(a7)
makevoidptr_out:
	rts
makevoidptr_string:
	bra	StringToChars

throw_Any:
	lea.l	throw_Any_msg1(pc),a0
	lea.l	throw_Any_msg2(pc),a1
	systrap	ErrDisplayFileLineMsg(a0.l,#0.w,a1.l)
	dc.w	breakpoint
	rts
throw_Any_msg1:	dc.b	'uncaught exception',0
throw_Any_msg2:	dc.b	'occurred',0
	align	2

;; dummy for unneeded methods, will never(?) be called
M_none:
	rts

;; JVM opcode 'new'
;; d0 = class index for new instance
;; returns new instance in (a7)
op_new:
	move.w	d0,d4
     	bsr	garbage
     	move.w	d4,d0
     	bsr	getclassinfo
     	clr.l	d3
     	move.w	ClassInfo.ObjectSizePlusHeader(a0),d3
     	systrap	MemPtrNew(d3.l)
     	lea	ObjectHeader_sizeof(a0),a2
     	systrap	MemSet(a0.l,d3.l,#0.b)
     	move.l	FirstObject(a5),ObjectHeader_NextObject(a2)
     	move.l	a2,FirstObject(a5)
     	move.w	d4,ObjectHeader_ClassIndex(a2)
     	move.l	(a7),a4
     	move.l	a2,(a7)
     	jmp	(a4)

;;; check instance for possible NullPointerException
;;; a0 = instance
nonnull_check:
	cmp.w	#0,a0
	beq	throw_NullPointerException
	rts
	
;;; check instance for possible NullPointerException
;;; called with (a7) = instance
nonnull_check_tos:
	tst.l	4(a7)		; skip return address
	beq	throw_NullPointerException
	rts
	
;;; check new array size for possible NegativeArraySizeException
;;; called with d3.l = size
posarray_check_d3:
	cmp.l	#$10000,d3
	bcc	throw_NegativeArraySizeException
	rts
	
;;; check array access for possible NullPointerException
;;; or ArrayIndexOutOfBoundsException
;;; a0 = array instance
;;; d0 = index
;;; uses d1
array_check:
	cmp.w	#0,a0
	beq	throw_NullPointerException
	clr.l	d1
	move.w	Array.length(a0),d1
	cmp.l	d1,d0
	bcc	throw_ArrayIndexOutOfBoundsException
	rts	

;;; check aastore for possible NullPointerException
;;; or ArrayIndexOutOfBoundsException
;;; or ArrayStoreException
;;; a0 = array instance
;;; d0 = index
;;; a1 = value
;;; uses d1, a2, a3
aastore_check:
	cmp.w	#0,a0
	beq	throw_NullPointerException
	clr.l	d1
	move.w	Array.length(a0),d1
	cmp.l	d1,d0
	bcc	throw_ArrayIndexOutOfBoundsException
	cmp.w	#0,a1
	beq.s	aastore_check_ok
	move.w	ObjectHeader_ClassIndex(a0),d1
	mulu.w	#ClassInfo_size,d1
	lea	ClassTable(pc),a2
	add.l	d1,a2
	move.w	ObjectHeader_ClassIndex(a1),d1
	mulu.w	#ClassInfo_size,d1
	lea	ClassTable(pc),a3
	add.l	d1,a3
	jsr	ClassInfo.AastoreCheck(a2)
	beq	throw_ArrayStoreException
aastore_check_ok:	
	rts	

;;; throw a newly-initialized NullPointerException
;;; (a7) = initiator address
;; uses-class java/lang/NullPointerException as NPEclass
;; uses-instance java/lang/NullPointerException 
;; uses-method java/lang/NullPointerException.<init>()V as NPEinit
throw_NullPointerException:
	move.l	#NPEclass,d0
	bsr	op_new
	bsr	NPEinit
	move.l	(a7)+,a2
	move.l	(a7)+,d2
	bra	do_athrow
	
;;; throw a newly-initialized ArrayIndexOutOfBoundsException
;;; (a7) = initiator address
;; uses-class java/lang/ArrayIndexOutOfBoundsException as AIOOBEclass
;; uses-instance java/lang/ArrayIndexOutOfBoundsException 
;; uses-method java/lang/ArrayIndexOutOfBoundsException.<init>()V as AIOOBEinit
throw_ArrayIndexOutOfBoundsException:
	move.l	#AIOOBEclass,d0
	bsr	op_new
	bsr	AIOOBEinit
	move.l	(a7)+,a2
	move.l	(a7)+,d2
	bra	do_athrow
	
;;; throw a newly-initialized ArrayStoreException
;;; (a7) = initiator address
;; uses-class java/lang/ArrayStoreException as ASEclass
;; uses-instance java/lang/ArrayStoreException 
;; uses-method java/lang/ArrayStoreException.<init>()V as ASEinit
throw_ArrayStoreException:
	move.l	#ASEclass,d0
	bsr	op_new
	bsr	ASEinit
	move.l	(a7)+,a2
	move.l	(a7)+,d2
	bra	do_athrow
	
;;; throw a newly-initialized ArithmeticException
;;; (a7) = initiator address
;; uses-class java/lang/ArithmeticException as AEclass
;; uses-instance java/lang/ArithmeticException 
;; uses-method java/lang/ArithmeticException.<init>()V as AEinit
throw_ArithmeticException:
	move.l	#AEclass,d0
	bsr	op_new
	bsr	AEinit
	move.l	(a7)+,a2
	move.l	(a7)+,d2
	bra	do_athrow

;;; throw a newly-initialized NegativeArraySizeException
;;; (a7) = initiator address
;; uses-class java/lang/NegativeArraySizeException as NASEclass
;; uses-instance java/lang/NegativeArraySizeException 
;; uses-method java/lang/NegativeArraySizeException.<init>()V as NASEinit
throw_NegativeArraySizeException:
	move.l	#NASEclass,d0
	bsr	op_new
	bsr	NASEinit
	move.l	(a7)+,a2
	move.l	(a7)+,d2
	bra	do_athrow
	
;; throw an exception
;; uses-method java/lang/Throwable.getMessage()Ljava/lang/String; as Throwable_getMessage
;; d2 = initiating address
;; a2 = exception instance
;; a6 = frame pointer of last full-featured stack frame
do_athrow:
        move.l  4(a6),a3	; a3 = address of Mxx__exceptions table
        cmp.l   #0,a3
        beq.s   do_athrow_skipframe	; no handler in this frame
        cmp.l   #-1,a3
        beq.s   do_athrow_notfound	; last frame reached
        lea     0(pc),a0
        sub.l   a0,d2
        sub.l   ExceptionTableHeader.method_offset(a3),d2
        move.w  ExceptionTableHeader.table_length(a3),d3
        move.l  a3,a1
        addq.l  #sizeof(ExceptionTableHeader),a1
        bra.s   do_athrow_searchend
		;; d2 = initiating offset
		;; d3 = loop count
		;; a1 = table pointer
do_athrow_searchloop:
		;; initiator within pc range of this entry?
        cmp.w   ExceptionTableEntry.start_pc_offset(a1),d2
        bls.s   do_athrow_searchnext
        cmp.w   ExceptionTableEntry.end_pc_offset(a1),d2
        bhi.s   do_athrow_searchnext
		;; Exception matches given type?
        move.w  ExceptionTableEntry.catch_type_index(a1),d0
        move.l  a2,a0
        bsr     instancecheck
        tst.l   d0
        beq.s   do_athrow_searchnext
		;; set sp = a6 - local_space
do_athrow_found:
        clr.l   d0
        move.w  ExceptionTableHeader.local_space(a3),d0
        move.l  a6,d1
        sub.l   d0,d1
        move.l  d1,a7
		;; push exception object
        move.l  a2,-(a7)
		;; branch to handler pc
        clr.l   d0
        move.w  ExceptionTableEntry.handler_pc_offset(a1),d0
        lea     0(pc),a0
        add.l   ExceptionTableHeader.method_offset(a3),a0
        add.l   d0,a0
        jmp     (a0)
		;; next entry in same frame
do_athrow_searchnext:
        addq.l  #sizeof(ExceptionTableEntry),a1
do_athrow_searchend:
        dbra    d3,do_athrow_searchloop
do_athrow_skipframe:
		;; next frame: d2 = initiator, a6 = fp
        move.l  8(a6),d2
        move.l  (a6),a6
        bra.s   do_athrow
do_athrow_notfound:
	bra	throw_Any
        move.w  ObjectHeader_ClassIndex(a2),d0
        bsr     getclassinfo
        lea     0(pc),a3
        add.l   ClassInfo.Name(a0),a3
        move.l  a2,-(a7)
        bsr     Throwable_getMessage
        move.l  (a0),a1
        systrap ErrDisplayFileLineMsg(a3.l, #0.w, Array.data(a1).l)
        dc.w    breakpoint
        rts

	
;; uses-method java/lang/String.<init>([C)V as String_init_$C_V
;; uses-class [C as class_$C
;; uses-class java/lang/String as class_String
;; uses-instance java/lang/String
;; uses-field java/lang/String.value as String_value
;; uses-field java/lang/String.count as String_count
;; uses-field java/lang/String.offset as String_offset
;; uses-method java/lang/String.toCharArray()[C as String_toCharArray
	;; -------- Pascal-style (length + chars) -> String
CharsToString:
	lea.l	DummyCharArray(a5),a1
	clr.l	ObjectHeader_LastPtr(a1)
	clr.l	ObjectHeader_NextObject(a1)
	move.w	#class_$C,ObjectHeader_ClassIndex(a1)
	move.w	#1,Array.elsize(a1)
	clr.w	Array.objtype(a1)
	move.w	(a0)+,Array.length(a1)
	move.l	a0,Array.data(a1)
	move.l	#class_String,d0
	bsr	op_new
	move.l	(a7),-(a7)
	lea.l	DummyCharArray(a5),a0
	move.l	a0,-(a7)
	bsr	String_init_$C_V
	lea.l	8(a7),a7
	move.l	(a7)+,a0
	rts
	;; -------- C-style (chars + '\0') -> String
CharPtr_to_String:
	move.l	a0,-(a7)
	systrap	StrLen(a0.l)
	move.l	d0,d3
	lea.l	DummyCharArray(a5),a2
	clr.l	ObjectHeader_LastPtr(a2)
	clr.l	ObjectHeader_NextObject(a2)
	move.w	#class_$C,ObjectHeader_ClassIndex(a2)
	move.w	#1,Array.elsize(a2)
	clr.w	Array.objtype(a2)
	move.l	(a7)+,Array.data(a2)
	move.w	d0,Array.length(a2)
	move.l	#class_String,d0	; Class java/lang/String
	bsr	op_new
	move.l	(a7),-(a7)
	lea.l	DummyCharArray(a5),a0
	move.l	a0,-(a7)
	bsr	String_init_$C_V	; Method java/lang/String.<init>([C)V
	lea.l	8(a7),a7
	move.l	(a7)+,a0
	rts
	;; -------- String -> C-style (chars + '\0')
	;; in:  a0 = Java String instance
	;; out:	a0 = (char *) C-string
StringToChars:
	move.l	String_value(a0),a1
	move.l	Array.data(a1),a1
	adda.l	String_offset(a0),a1
	move.l	String_count(a0),d0
	tst.b	0(a1,d0.l)
	bne.s	StringToChars_xxl
	move.l	a1,a0
	rts
StringToChars_xxl:
	move.l	a0,-(a7)
	bsr	String_toCharArray
	addq.l	#4,a7
	move.l	Array.data(a0),a0
	rts
	
	
;; skeleton character array, used in different places
	data
	ds.l	2
	ds.w	1
DummyCharArray:	ds.w	3
	ds.l	1
	code

;; garbage collector
garbage:
		;; scratch regs d0, d1, a0, a1, a2, a3
		;; get beginning of object chain
        lea     FirstObject(a5),a2
gc_loop:
		;; NULL pointer means end-of-chain
        move.l  (a2),a1
        cmp.l   #0,a1
        beq     gc_out
		;; addr of a ptr that once pointed here
        move.l  ObjectHeader_LastPtr(a1),a0
		;; rk: inserted check - no valid pointer?
	cmp.l	#0,a0
	beq	gc_must_search
		;; still pointing here?
        cmp.l   (a0),a1
        beq     gc_next
gc_must_search:	
        move.l  a7,a0
        move.l  StackEnd(a5),d0
        sub.l   a0,d0
        lsr.w   #1,d0
		;; avoid reaching out beyond the stack's area
	subq.l	#1,d0
        moveq.l #2,d1
		;; search through all stack entries (align 2)
        bsr     gc_search
        cmp.l   #0,a0
        bne.s   gc_found
        lea     StaticObjects(a5),a0
        move.w  StaticObjectCount(pc),d0
        moveq.l #4,d1
 		;; search through all static objects
	bsr.s   gc_search
        cmp.l   #0,a0
        bne.s   gc_found
        move.l  FirstObject(a5),a3
gc_obj_loop:
		;; end of object chain?
        cmp.l   #0,a3
        beq.s   gc_obj_out
		;; ignore self-references
        cmp.l   a3,a1
        beq.s   gc_obj_next
        move.w  ObjectHeader_ClassIndex(a3),d0
        bsr     getclassinfo
		;; is an object array?
        move.w  ClassInfo.Flags(a0),d0
        and.w   #ClassInfo_arrayofobjects,d0
        bne.s   gc_obj_array
        move.w  ClassInfo.ObjectSizePlusHeader(a0),d0
		;; subtract 2 more to avoid the last access
		;; otherwise extending beyond the instance's end.
        sub.w   #ObjectHeader_sizeof+2,d0
	bcs.s	gc_obj_next
        lsr.w   #1,d0
        moveq.l #2,d1
        move.l  a3,a0
		;; search the fields of object
        bsr.s   gc_search
        bra     gc_obj_check
gc_obj_array:
        move.w  Array.length(a3),d0
        moveq.l #4,d1
        move.l  Array.data(a3),a0
		;; search through array elements
        bsr.s   gc_search
gc_obj_check:
        cmp.l   #0,a0
        bne.s   gc_found
gc_obj_next:
        move.l  ObjectHeader_NextObject(a3),a3
        bra     gc_obj_loop
		;; no reference to object found - reclaim it
gc_obj_out:
        move.l  ObjectHeader_NextObject(a1),(a2)
        bsr.s   gc_zap
        bra     gc_loop
gc_found:
        move.l  a0,ObjectHeader_LastPtr(a1)
gc_next:
        move.l  (a2),a2
        lea     ObjectHeader_NextObject(a2),a2
        bra     gc_loop
gc_out:
        rts
		;; linear (array) search
		;; d0 = count
	        ;; d1 = search address increment
	        ;; a0 = beg. of pointer array
	        ;; a1 = object to search for
gc_search_loop:
        cmp.l   (a0),a1
        beq.s   gc_search_out
        add.l   d1,a0
gc_search:
        dbra    d0,gc_search_loop
        move.l  #0,a0
gc_search_out:
        rts
		;; reclaim object
gc_zap:
        move.w  ObjectHeader_ClassIndex(a1),d0
        bsr     getclassinfo
        move.w  ClassInfo.Flags(a0),d1
        and.w   #ClassInfo_array,d1
        beq.s   gc_zap_notarray
        movem.l a0/a1,-(a7)
        systrap MemChunkFree(Array.data(a1).l)
        movem.l (a7)+,a0/a1
gc_zap_notarray:
        clr.l   d0
        move.w  ClassInfo.ObjectSizePlusHeader(a0),d0
        lea     -ObjectHeader_sizeof(a1),a3
        systrap MemSet(a3.l,d0.l,#205.b)
        systrap MemChunkFree(a3.l)
        rts

