;;
;; the routines for Jump2 that implement JVM opcodes
;;

;; jvm-opcode newarray
; d0 = class index, d1 = element size, 4(a7) = length
	move.w	d1,-(a7)	; save elsize on stack
	bsr	op_new
	move.l	(a7)+,a3	; array instance
	move.w	(a7)+,d3
	move.l	4(a7),d0	; get length from stack
	cmp.l	#$10000,d0
	bcc	throw_NegativeArraySizeException
	move.w	d0,Array.length(a3)
	move.w	d3,Array.elsize(a3)
	move.l	a3,4(a7)	; insert result into stack
	mulu.w	d0,d3
	addq.l	#1,d3		; get 1 extra byte for a '\0'
	systrap	MemPtrNew(d3.l)
	move.l	a0,Array.data(a3)
	systrap	MemSet(a0.l,d3.l,#0.b)
	rts

;; jvm-opcode anewarray
; d0 = class index, 4(a7) = length
	bsr	op_new
	move.l	(a7)+,a3	; array instance
	move.l	4(a7),d3	; get length from stack
	cmp.l	#$10000,d3
	bcc	throw_NegativeArraySizeException
	move.w	d3,Array.length(a3)
	move.w	#4,Array.elsize(a3)
	move.l	a3,4(a7)	; insert result into stack
	asl.l	#2,d3
	systrap	MemPtrNew(d3.l)
	move.l	a0,Array.data(a3)
	systrap	MemSet(a0.l,d3.l,#0.b)
	rts

;; jvm-opcode imul
	move.l	4(a7),d0	; d0 = factor1
	move.l	8(a7),d1	; d1 = factor2
	move.w	d0,d2
	mulu.w	d1,d2		; d2 = LL
	move.l	d0,d3
	swap.w	d3		; d3 = swap(factor1)
	mulu.w	d1,d3		; d3 = HL
	swap.w	d3
	clr.w	d3
	add.l	d3,d2		; d2 = LL + HL<<16
	move.l	d1,d3
	swap.w	d3		; d3 = swap(factor2)
	mulu.w	d0,d3		; d3 = LH
	swap.w	d3
	clr.w	d3
	add.l	d3,d2		; d2 = LL + HL<<16 + LH<<16
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d2,(a7)
     	jmp	(a4)

;; jvm-opcode idiv
;; uses-class java/lang/ArithmeticException as class_AE
;; uses-instance java/lang/ArithmeticException
;; uses-method java/lang/ArithmeticException.<init>()V as method_AE_init
	moveq.l	#1,d2		; sign flag
	move.l	4(a7),d0	; denominator
	beq	op_idiv_throw	; (a7) = initiating address
	bpl.s	op_idiv_a
	neg.l	d0
	neg.b	d2
op_idiv_a:			; d0 = abs(den)
	move.l	8(a7),d1	; numerator
	bpl.s	op_idiv_b
	neg.l	d1
	neg.b	d2
op_idiv_b:			; d1 = abs(num)
	cmp.l	#$10000,d0
	bcc.s	op_idiv_32	; divide 32 by 32 bit
	cmp.l	#$10000,d1
	bcc.s	op_idiv_32_16	; divide 32 by 16 bit
	; --- divide 16 by 16 bit ---
	divu.w	d0,d1		; using 16-bit division
	clr.l	d0
	move.w	d1,d0		; d0 = abs(result)
	tst.b	d2
	bpl.s	op_idiv_c
	neg.l	d0
op_idiv_c:
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d0,(a7)
     	jmp	(a4)
	;
	; --- divide 32 by 16 bit ---
op_idiv_32_16:
	move.l	d1,d3
	clr.w	d3
	swap.w	d3		; d3.w = high(num)
	divu.w	d0,d3
	clr.l	d4
	move.w	d3,d4
	swap.w	d4		; d4 = high(num/den)
	move.w	d1,d3		; d3 = high-remainder<<16 + low(num)
	divu.w	d0,d3
	move.w	d3,d4		; d4 = abs(num/den)
	tst.b	d2
	bpl.s	op_idiv_d
	neg.l	d4
op_idiv_d:
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d4,(a7)
     	jmp	(a4)
	;
	; --- divide 32 by 32 bit: d1/d0 ---
	; we know that the denominator is greater than $FFFF.
op_idiv_32:
	clr.l	d4
	move.l	#$8000,d5	; add-value for quotient (first iteration)
	move.l	d1,d3
	clr.w	d3
	swap.w	d3		; d3 = high(num)
op_idiv_loop:
	;; d5 = result bit mask
	;; d4 = result
	;; d3 = shifted numerator
	lsl.w	#1,d1
	roxl.l	#1,d3
	cmp.l	d0,d3
	bcs.s	op_idiv_e
	add.l	d5,d4		; set quotient bit
	sub.l	d0,d3		; adjust numerator
op_idiv_e:
	lsr.l	#1,d5
	bne.s	op_idiv_loop	; repeat if add-value > 0
	tst.b	d2
	bpl.s	op_idiv_f
	neg.l	d4
op_idiv_f:
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d4,(a7)
     	jmp	(a4)
op_idiv_throw:
	move.l	#class_AE,d0
	bsr	op_new
	bsr	method_AE_init
	move.l	(a7)+,a2	; exception instance
	move.l	(a7)+,d2	; initiator address
	bra	do_athrow

;; jvm-opcode irem
;; uses-class java/lang/ArithmeticException as class_AE
;; uses-instance java/lang/ArithmeticException
;; uses-method java/lang/ArithmeticException.<init>()V as method_AE_init
	moveq.l	#1,d2		; sign flag
	move.l	4(a7),d0	; denominator
	beq	op_irem_throw
	bpl.s	op_irem_a
	neg.l	d0
	neg.b	d2
op_irem_a:			; d0 = abs(den)
	move.l	8(a7),d1	; numerator
	bpl.s	op_irem_b
	neg.l	d1
op_irem_b:			; d1 = abs(num)
	cmp.l	#$10000,d0
	bcc.s	op_irem_32	; divide 32 by 32 bit
	cmp.l	#$10000,d1
	bcc.s	op_irem_32_16	; divide 32 by 16 bit
	; --- divide 16 by 16 bit ---
	divu.w	d0,d1		; using 16-bit division
	clr.l	d0
	swap.w	d1
	move.w	d1,d0		; d0 = abs(result)
	tst.b	d2
	bpl.s	op_irem_c
	neg.l	d0
op_irem_c:
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d0,(a7)
     	jmp	(a4)
	;
	; --- divide 32 by 16 bit ---
op_irem_32_16:
	move.l	d1,d3
	clr.w	d3
	swap.w	d3		; d3.w = high(num)
	divu.w	d0,d3
	move.w	d1,d3		; d3 = high-remainder<<16 + low(num)
	divu.w	d0,d3
	clr.l	d4
	swap.w	d3
	move.w	d3,d4		; d4 = abs(num/den)
	tst.b	d2
	bpl.s	op_irem_d
	neg.l	d4
op_irem_d:
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d4,(a7)
     	jmp	(a4)
	;
	; --- divide 32 by 32 bit ---
	; we know that the denominator is greater than $FFFF.
op_irem_32:
	clr.l	d4
	move.l	#$8000,d5	; add-value for quotient (first iteration)
	move.l	d1,d3
	clr.w	d3
	swap.w	d3		; d3 = high(num)
op_irem_loop:
	lsl.w	#1,d1
	roxl.l	#1,d3
	cmp.l	d0,d3
	bcs.s	op_irem_e
	sub.l	d0,d3		; adjust numerator
op_irem_e:
	lsr.l	#1,d5
	bne.s	op_irem_loop	; repeat if add-value > 0
	tst.b	d2
	bpl.s	op_irem_f
	neg.l	d3
op_irem_f:
	move.l	(a7),a4
	addq	#8,a7
     	move.l	d3,(a7)
     	jmp	(a4)
op_irem_throw:
	move.l	#class_AE,d0
	bsr	op_new
	bsr	method_AE_init
	move.l	(a7)+,a2	; exception instance
	move.l	(a7)+,d2	; initiator address
	bra	do_athrow

;; jvm-opcode lshl
	move.l	8(a7),d0
	move.l	12(a7),d1
	move.l	4(a7),d2
	beq.s	op_lshl_out
	btst.l	#5,d2		; shift 32 or more?
	bne.s	op_lshl_32
	; shift by less than 32 bits: use both registers
	moveq.l	#-1,d3
	asl.l	d2,d3		; bit mask
	asl.l	d2,d0
	rol.l	d2,d1
	move.l	d1,d2
	and.l	d3,d1		; make low bits = 0
	not.l	d3
	and.l	d3,d2		; isolate carry bits
	add.l	d2,d0		; add carry bits
	bra.s	op_lshl_out
	; shift more than 32 bits: single-register case
op_lshl_32:
	move.l	d1,d0
	clr.l	d1
	andi.w	#$001f,d2
	asl.l	d2,d0
op_lshl_out:
	move.l	(a7)+,a4
	addq.l	#4,a7
	move.l	d0,(a7)
	move.l	d1,4(a7)
	jmp	(a4)

;; jvm-opcode lushr
	move.l	8(a7),d0
	move.l	12(a7),d1
	move.l	4(a7),d2
	beq.s	op_lushr_out
	btst.l	#5,d2		; shift 32 or more?
	bne.s	op_lushr_32
	; shift by less than 32 bits: use both registers
	moveq.l	#-1,d3
	lsr.l	d2,d3		; bit mask
	ror.l	d2,d0
	lsr.l	d2,d1
	move.l	d0,d2
	and.l	d3,d0		; make high bits = 0
	not.l	d3
	and.l	d3,d2		; isolate carry bits
	add.l	d2,d1		; add carry bits
	bra.s	op_lushr_out
	; shift more than 32 bits: single-register case
op_lushr_32:
	move.l	d0,d1
	clr.l	d0
	andi.w	#$001f,d2
	lsr.l	d2,d1
op_lushr_out:
	move.l	(a7)+,a4
	addq.l	#4,a7
	move.l	d0,(a7)
	move.l	d1,4(a7)
	jmp	(a4)

;; jvm-opcode lshr
	move.l	8(a7),d0
	move.l	12(a7),d1
	move.l	4(a7),d2
	beq.s	op_lshr_out
	btst.l	#5,d2		; shift 32 or more?
	bne.s	op_lshr_32
	; shift by less than 32 bits: use both registers
	moveq.l	#-1,d3
	lsr.l	d2,d3		; bit mask
	ror.l	d2,d0
	lsr.l	d2,d1
	move.l	d0,d2
	and.l	d3,d0		; make high bits = 0
	not.l	d3
	and.l	d3,d2		; isolate carry bits
	add.l	d2,d1		; add carry bits
	tst.w	8(a7)		; check operand sign
	bpl.s	op_lshr_out
	add.l	d3,d0		; add sign-extend bits
	bra.s	op_lshr_out
	; shift more than 32 bits: single-register case
op_lshr_32:
	move.l	d0,d1
	smi.b	d0
	ext.w	d0
	ext.l	d0
	andi.w	#$001f,d2
	asr.l	d2,d1
op_lshr_out:
	move.l	(a7)+,a4
	addq.l	#4,a7
	move.l	d0,(a7)
	move.l	d1,4(a7)
	jmp	(a4)

;; jvm-opcode lmul
	movem.l	d4-d7,-(a7)
	move.l	20(a7),d4	; d4 = factor1H = d.c
	move.l	24(a7),d5	; d5 = factor1L = b.a
	move.l	28(a7),d6	; d6 = factor2H = D.C
	move.l	32(a7),d7	; d7 = factor2L = B.A
	;
	move.w	d4,d0
	mulu.w	d7,d0		; (1) d0 = c * A (in 32-63)
	;
	move.w	d5,d1
	mulu.w	d7,d1		; (2) d1 = a * A (in 00-31)
	;
	swap.w	d5		; d5 = inverse = a.b
	move.w	d5,d2
	mulu.w	d7,d2		; (3) d2 = b * A
	swap.w	d2
	clr.l	d3
	move.w	d2,d3
	clr.w	d2
	add.l	d2,d1
	addx.w	d3,d0		; added to d0/d1 (16-47)
	;
	swap.w	d7		; d7 = inverse = A.B
	move.w	d7,d2
	mulu.w	d5,d2		; (4) d2 = b * B
	add.l	d2,d0		; added to d0/d1 (32-63)
	;
	swap.w	d5		; d5 = normal = b.a
	move.w	d5,d2
	mulu.w	d7,d2		; (5) d2 = a * B
	swap.w	d2
	clr.l	d3
	move.w	d2,d3
	clr.w	d2
	add.l	d2,d1
	addx.w	d3,d0		; added to d0/d1 (16-47)
	;
	move.w	d4,d2
	mulu.w	d7,d2		; (6) d2 = c * B
	swap.w	d2
	clr.w	d2
	add.l	d2,d0		; added to d0/d1 (48-63)
	;
	move.w	d5,d2
	mulu.w	d6,d2		; (7) d2 = a * C
	add.l	d2,d0		; added to d0/d1 (32-63)
	;
	swap.w	d4		; d4 = inverse = c.d
	swap.w	d7		; d7 = normal = B.A
	move.w	d4,d2
	mulu.w	d7,d2		; (8) d2 = d * A
	swap.w	d2
	clr.w	d2
	add.l	d2,d0		; added to d0/d1 (48-63)
	;
	swap.w	d5		; d5 = invers = a.b
	move.w	d5,d2
	mulu.w	d6,d2		; (9) d2 = b * C
	swap.w	d2
	clr.w	d2
	add.l	d2,d0		; added to d0/d1 (48-63)
	;
	swap.w	d5		; d5 = normal = b.a
	swap.w	d6		; d6 = invers = C.D
	move.w	d5,d2
	mulu.w	d6,d2		; (10) d2 = a * D
	swap.w	d2
	clr.w	d2
	add.l	d2,d0		; added to d0/d1 (48-63)
	;
	movem.l	(a7)+,d4-d7
	move.l	(a7)+,a4
	addq	#8,a7		; one long popped from stack
     	move.l	d0,(a7)
	move.l	d1,4(a7)
     	jmp	(a4)

;; jvm-opcode ldiv
;; uses-class java/lang/ArithmeticException as class_AE
;; uses-instance java/lang/ArithmeticException
;; uses-method java/lang/ArithmeticException.<init>()V as method_AE_init
	movem.l	d4-d7,-(a7)
	move.l	20(a7),d4
	move.l	24(a7),d5	; d4,d5 = denominator
	move.l	28(a7),d6
	move.l	32(a7),d7	; d6,d7 = numerator
	tst.l	d4
	bne.s	op_ldiv_nz
	tst.l	d5
	bne.s	op_ldiv_nz
	;
	; division by zero: throw exception
	;
	movem.l	(a7)+,d4-d7	; now (a7) is exception-initiating PC
	move.l	#class_AE,d0
	bsr	op_new
	bsr	method_AE_init
	move.l	(a7)+,a2	; exception instance
	move.l	(a7)+,d2	; initiator address
	bra	do_athrow
op_ldiv_nz:
	tst.l	d4
	bpl.s	op_ldiv_a
	neg.l	d5
	negx.l	d4
op_ldiv_a:			; d4,d5 = abs(den)
	tst.l	d6
	bpl.s	op_ldiv_b
	neg.l	d7
	negx.l	d6
op_ldiv_b:			; d6,d7 = abs(num)
	tst.l	d4
	bne	op_ldiv_64
	;
	; divide 64 by 32: compute high lword
	; numerator = #0,d3 . d6,d7 ('.' is decimal point)
	; denominator = #0,d5
	; result bit mask = d2
	; result = d0,d1 (first filling d0)
	clr.l	d3
	clr.l	d0
	clr.l	d1
	moveq.l	#1,d2
op_ldiv_c:
	ror.l	#1,d2
	move.w	#0,ccr
	roxl.l	#1,d7
	roxl.l	#1,d6
	roxl.l	#1,d3
	cmp.l	d5,d3
	bcs.s	op_ldiv_d
	; found a 1 bit for the result
	sub.l	d5,d3
	add.l	d2,d0
op_ldiv_d:
	cmp.l	#1,d2
	bne.s	op_ldiv_c
	;
	; continue to divide 64 by 32: compute low lword
	; numerator = C-flag,d3 . d6 ('.' is decimal point)
	; denominator = #0,d5
	; result bit mask = d2
	; result = d0,d1 (now filling d1)
	clr.l	d7
op_ldiv_e:
	ror.l	#1,d2
	move.w	#0,ccr
	roxl.l	#1,d6
	roxl.l	#1,d3
	bcs.s	op_ldiv_e1	; C-flag set: number > denominator
	cmp.l	d5,d3
	bcs.s	op_ldiv_f
op_ldiv_e1:
	; found a 1 bit for the result
	sub.l	d5,d3
	add.l	d2,d1
op_ldiv_f:
	cmp.l	#1,d2
	bne.s	op_ldiv_e
	bra	op_ldiv_out
	;
	; divide 64 by 64: result only has low lword
	; numerator = d3,d6 . d7 ('.' is decimal point)
	; denominator = d4,d5
	; result bit mask = d2
	; result = #0,d1
op_ldiv_64:
	clr.l	d3
	clr.l	d0
	clr.l	d1
	moveq.l	#1,d2
op_ldiv_j:
	ror.l	#1,d2
	move.w	#0,ccr
	roxl.l	#1,d7
	roxl.l	#1,d6
	roxl.l	#1,d3
	cmp.l	d4,d3
	bcs.s	op_ldiv_k	; definitely too small
	bne.s	op_ldiv_l	; definitely bigger
	cmp.l	d5,d6		; high lwords equal: compare low lwords
	bcs.s	op_ldiv_k
op_ldiv_l:
	; found a 1 bit for the result
	sub.l	d5,d6
	subx.l	d4,d3
	add.l	d2,d0
op_ldiv_k:
	cmp.l	#1,d2
	bne.s	op_ldiv_j
	;
op_ldiv_out:
	; give the correct sign to the result
	tst.w	20(a7)
	bpl.s	op_ldiv_g
	tst.w	28(a7)
	bmi.s	op_ldiv_h
	bra.s	op_ldiv_i
op_ldiv_g:
	tst.w	28(a7)
	bpl.s	op_ldiv_h
op_ldiv_i:
	neg.l	d1
	negx.l	d0
op_ldiv_h:			; result sign O.K. now
	move.l	d0,28(a7)
	move.l	d1,32(a7)
	movem.l	(a7)+,d4-d7
	move.l	(a7)+,a4
	addq.l	#8,a7
	jmp	(a4)

;; jvm-opcode lrem
;; uses-class java/lang/ArithmeticException as class_AE
;; uses-instance java/lang/ArithmeticException
;; uses-method java/lang/ArithmeticException.<init>()V as method_AE_init
	movem.l	d4-d7,-(a7)
	move.l	20(a7),d4
	move.l	24(a7),d5	; d4,d5 = denominator
	move.l	28(a7),d6
	move.l	32(a7),d7	; d6,d7 = numerator
	tst.l	d4
	bne.s	op_lrem_nz
	tst.l	d5
	bne.s	op_lrem_nz
	;
	; division by zero: throw exception
	;
	movem.l	(a7)+,d4-d7	; now (a7) is exception-initiating PC
	move.l	#class_AE,d0
	bsr	op_new
	bsr	method_AE_init
	move.l	(a7)+,a2	; exception instance
	move.l	(a7)+,d2	; initiator address
	bra	do_athrow
op_lrem_nz:
	tst.l	d4
	bpl.s	op_lrem_a
	neg.l	d5
	negx.l	d4
op_lrem_a:			; d4,d5 = abs(den)
	tst.l	d6
	bpl.s	op_lrem_b
	neg.l	d7
	negx.l	d6
op_lrem_b:			; d6,d7 = abs(num)
	tst.l	d4
	bne	op_lrem_64
	;
	; divide 64 by 32: compute high lword
	; numerator = #0,d3 . d6,d7 ('.' is decimal point)
	; denominator = #0,d5
	; result bit mask = d2
	clr.l	d3
	clr.l	d0
	clr.l	d1
	moveq.l	#1,d2
op_lrem_c:
	ror.l	#1,d2
	move.w	#0,ccr
	roxl.l	#1,d7
	roxl.l	#1,d6
	roxl.l	#1,d3
	cmp.l	d5,d3
	bcs.s	op_lrem_d
	; found a 1 bit for the result
	sub.l	d5,d3
op_lrem_d:
	cmp.l	#1,d2
	bne.s	op_lrem_c
	;
	; continue to divide 64 by 32: compute low lword
	; numerator = C-flag,d3 . d6 ('.' is decimal point)
	; denominator = #0,d5
	; result bit mask = d2
	clr.l	d7
op_lrem_e:
	ror.l	#1,d2
	move.w	#0,ccr
	roxl.l	#1,d6
	roxl.l	#1,d3
	bcs.s	op_lrem_e1
	cmp.l	d5,d3
	bcs.s	op_lrem_f
op_lrem_e1:
	; found a 1 bit for the result
	sub.l	d5,d3
op_lrem_f:
	cmp.l	#1,d2
	bne.s	op_lrem_e
	clr.l	d0
	move.l	d3,d1
	bra	op_lrem_out
	;
	; divide 64 by 64: result only has low lword
	; numerator = d3,d6 . d7 ('.' is decimal point)
	; denominator = d4,d5
	; result bit mask = d2
op_lrem_64:
	clr.l	d3
	clr.l	d0
	clr.l	d1
	moveq.l	#1,d2
op_lrem_j:
	ror.l	#1,d2
	move.w	#0,ccr
	roxl.l	#1,d7
	roxl.l	#1,d6
	roxl.l	#1,d3
	cmp.l	d4,d3
	bcs.s	op_lrem_k	; definitely too small
	bne.s	op_lrem_l	; definitely bigger
	cmp.l	d5,d6		; high lwords equal: compare low lwords
	bcs.s	op_lrem_k
op_lrem_l:
	; found a 1 bit for the result
	sub.l	d5,d6
	subx.l	d4,d3
	add.l	d2,d0
op_lrem_k:
	cmp.l	#1,d2
	bne.s	op_lrem_j
	move.l	d3,d0
	move.l	d6,d1
	;
op_lrem_out:
	; give the correct sign to the result
	tst.w	28(a7)
	bpl.s	op_lrem_h
	neg.l	d1
	negx.l	d0
op_lrem_h:			; result sign O.K. now
	move.l	d0,28(a7)
	move.l	d1,32(a7)
	movem.l	(a7)+,d4-d7
	move.l	(a7)+,a4
	addq.l	#8,a7
	jmp	(a4)

;; jvm-opcode monitorenter
	rts

;; jvm-opcode monitorexit
	rts

;; jvm-opcode athrow
        move.l  (a7)+,d2	; d2 = initiating address
        move.l  (a7)+,a2	; a2 = exception object
	cmp.w	#0,a2
	bne	do_athrow
	move.l	d2,-(a7)
	bra	throw_NullPointerException
	

; --- (geo) patch begin ---

	
;; jvm-opcode d2l
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_dtoll,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode f2l
	move.l	a6,-(a7)
	move.l  a7,a6
	subq.l  #8,a7
	move.l	#sysFloatEm_f_ftoll,d2
	move.l	8(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode f2d
	move.l	a6,-(a7)
	move.l  a7,a6
	subq.l  #8,a7
	move.l	#sysFloatEm_f_ftod,d2
	move.l	8(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode f2i
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_ftoi,d2
	move.l	8(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fneg
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_neg,d2
	move.l	8(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fcmpg
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_cmp,d2
	move.l	8(a6),-(a7)
	move.l	12(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d0,d1	
	cmp.l	#flpEqual,d1
	beq	op_fcmpg_done
	move.l  #-1,d0
	cmp.l   #flpLess,d1
	beq	op_fcmpg_done
	move.l  #1,d0
op_fcmpg_done:	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fcmpl
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_cmp,d2
	move.l	8(a6),-(a7)
	move.l	12(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d0,d1	
	cmp.l	#flpEqual,d1
	beq	op_fcmpl_done
	move.l  #1,d0
	cmp.l   #flpGreater,d1
	beq	op_fcmpl_done
	move.l  #-1,d0
op_fcmpl_done:	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fadd
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_add,d2
	move.l	8(a6),-(a7)
	move.l	12(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fsub
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_sub,d2
	move.l	8(a6),-(a7)
	move.l	12(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fmul
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_mul,d2
	move.l	8(a6),-(a7)
	move.l	12(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode fdiv
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_div,d2
	move.l	8(a6),-(a7)
	move.l	12(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode d2f
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_d_dtof,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode d2i
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_d_dtoi,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode dneg
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_neg,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode dcmpg
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_d_cmp,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	move.l	20(a6),-(a7)
	move.l	16(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d0,d1	
	cmp.l	#flpEqual,d1
	beq	op_dcmpg_done
	move.l  #-1,d0
	cmp.l   #flpLess,d1
	beq	op_dcmpg_done
	move.l  #1,d0
op_dcmpg_done:	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode dcmpl
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_d_cmp,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	move.l	20(a6),-(a7)
	move.l	16(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d0,d1	
	cmp.l	#flpEqual,d1
	beq	op_dcmpl_done
	move.l  #1,d0
	cmp.l   #flpGreater,d1
	beq	op_dcmpl_done
	move.l  #-1,d0
op_dcmpl_done:	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode dadd
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_add,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	move.l	20(a6),-(a7)
	move.l	16(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode dsub
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_sub,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	move.l	20(a6),-(a7)
	move.l	16(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode dmul
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_mul,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	move.l	20(a6),-(a7)
	move.l	16(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode ddiv
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_div,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	move.l	20(a6),-(a7)
	move.l	16(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode frem
	move.l	a6,-(a7)        ; y,x -> (y-(y/x)*x)
	move.l  a7,a6
        move.l  12(a6),-(a7)	; y
	move.l  8(a6),-(a7)     ; x
	move.l  12(a6),-(a7)	; y
	move.l  8(a6),-(a7)     ; x
	move.l	#sysFloatEm_fp_round,d2
	systrap	FlpEmDispatch (#flpTowardZero.l)
	bsr	op_fdiv
	move.l	#sysFloatEm_fp_round,d2
	systrap	FlpEmDispatch (#flpToNearest.l)
	bsr	op_fmul
	bsr	op_fsub
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7	
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode drem
	move.l	a6,-(a7)
	move.l  a7,a6
        move.l  20(a6),-(a7)	; y
	move.l  16(a6),-(a7)	; y
	move.l  12(a6),-(a7)    ; x
	move.l  8(a6),-(a7)     ; x
        move.l  20(a6),-(a7)	; y
	move.l  16(a6),-(a7)	; y
	move.l  12(a6),-(a7)    ; x
	move.l  8(a6),-(a7)     ; x
	move.l	#sysFloatEm_fp_round,d2
	systrap	FlpEmDispatch (#flpTowardZero.l)
	bsr	op_ddiv
	move.l	#sysFloatEm_fp_round,d2
	systrap	FlpEmDispatch (#flpToNearest.l)
	bsr	op_dmul
	bsr	op_dsub
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	lea.l	 12(a7),a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode l2f
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_lltof,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode l2d
	move.l	a6,-(a7)
	move.l  a7,a6
	sub.l   #8,a7	
	move.l	#sysFloatEm_d_lltod,d2
	move.l	12(a6),-(a7)
	move.l	8(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	addq.l  #4,a7
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

;; jvm-opcode i2f
	move.l	a6,-(a7)
	move.l  a7,a6
	move.l	#sysFloatEm_f_itof,d2
	move.l	8(a6),-(a7)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	move.l  d0,(a7)
	jmp	(a4)

;; jvm-opcode i2d
	move.l	a6,-(a7)
	move.l  a7,a6
	subq.l  #8,a7
	move.l	#sysFloatEm_d_itod,d2
	move.l	8(a6),-(a7)
	pea	-8(a6)
	trap	#15
	dc.w	sysTrapFlpEmDispatch
	move.l  a6,a7
	move.l  (a7)+,a6
	move.l  (a7)+,a4
	move.l  d1,(a7)
	move.l  d0,-(a7)
	jmp	(a4)

; --- (geo) patch end ---

