package tools;

import java.io.*;
import java.util.*;
import ASSERT;

class ParamInfo {
  String type;
  String name;
  int    num;
}

class MkApi {
  
  // orig, java, signature [, PalmOS signature]

  /**
   * Translation table for PalmOS return-types.
   * <ul>
   * <li>0: original return-type (from PalmOS .h files)</li>
   * <li>1: Java-return-type to be used in generated Java stub</li>
   * <li>2: type-signature for Java-return-type</li>
   * <li>3: (optional) type-signature from PalmOS point-of-view,
   *        needed to generate conversion code (sign-extension etc.),
   *        defaults to Java type-signature.</li>
   * </ul>
   */

  static final String RET_XLATE[][] = 
  {{"DmOpenRef",         "int",     "I", "L"},
   {"FieldPtr",          "int",     "I", "L"},
   {"FontPtr",           "int",     "I", "L"},
   {"FormPtr",           "int",     "I", "L"},
   {"MenuBarPtr",        "int",     "I", "L"},
   {"VoidHand",          "int",     "I", "L"},
   {"VoidPtr",           "int",     "I", "L"},
   {"WinHandle",         "int",     "I", "L"},
   {"WinPtr",            "int",     "I", "L"},
   {"Err",               "int",     "I", "S+"},
   {"FontID",            "int",     "I", "B+"},
   {"FormObjectKind",    "int",     "I", "B+"},
   {"LocalIDKind",       "int",     "I", "B+"},
   {"UnderlineModeType", "int",     "I", "B+"},
   {"short",             "int",     "I", "S"},
   {"Boolean",           "boolean", "Z"},
   {"Byte",              "byte",    "B"},
   {"CharPtr",           "String",  "Ljava/lang/String;"},
   {"DWord",             "int",     "I"},
   {"Handle",            "int",     "I", "L"},
   {"int",               "int",     "I", "S"},
   {"Int",               "int",     "I", "S"},
   {"LocalID",           "int",     "I"},
   {"UInt",              "int",     "I", "S+"},
   {"ULong",             "int",     "I"},
   {"void",              "void",    "V"},
   {"Word",              "int",     "I", "S+"}
  };

  // orig, java, signature [, PalmOS signature]
  static final String PARAM_XLATE[][] = 
  {
   {"BitmapPtr",        "int",           "I"},
   {"Boolean",          "boolean",       "Z", "S"},
   {"Boolean*",         "Boolean",       "Ljava/lang/Boolean;"},
   {"BooleanPtr",       "Boolean",       "Ljava/lang/Boolean;"},
   {"byte",             "int",           "I", "B"},
   {"Byte",             "int",           "I", "B"},
   {"ByteArray",        "byte[]",        "[B"},
   {"char",             "char",          "C", "S"},
   {"CharBuf",          "StringBuffer",  "Ljava/lang/StringBuffer;"},
   {"CharPtr",          "String",        "Ljava/lang/String;"},
   {"ClipboardFormatType", "int",        "I", "B"},
   {"ControlPtr",       "int",           "I"},
   {"DateFormatType",   "int",           "I", "B"},
   {"DatePtr",          "Date",          "Lpalmos/Date;"},
   {"DateTimePtr",      "DateTime",      "Lpalmos/DateTime;"},
   {"DateType",         "int",           "I", "S"},
   {"DaySelectorPtr",   "DaySelector",   "Lpalmos/DaySelector;"},
   {"DirectionType",    "int",           "I", "B"},
   {"DmOpenRef",        "int",           "I"},
   {"DmOpenRef*",       "Integer",       "Ljava/lang/Integer;"},
   {"DmSearchStatePtr", "DmSearchState", "Lpalmos/DmSearchState;"},
   {"DWord",            "int",           "I"},
   {"DWord*",           "Integer",       "Ljava/lang/Integer;"},
   {"DWordPtr",         "Integer",       "Ljava/lang/Integer;"},
   {"EventPtr",         "Event",         "Lpalmos/Event;"},
   {"FieldPtr",         "int",           "I"},
   {"FieldAttrPtr",     "FieldAttr",     "Lpalmos/FieldAttr;"},
   {"FindParamsPtr",    "FindParams",    "Lpalmos/FindParams;"},
   {"FontID",           "int",           "I", "B"},
   {"FormPtr",          "int",           "I"},
   {"FrameType",        "int",           "I", "S"},
   {"GrfMatchInfoPtr",  "GrfMatchInfo",  "Lpalmos/GrfMatchInfo;"},
   {"Handle",           "int",           "I"},
   {"Handle*",          "Integer",       "Ljava/lang/Integer;"},
   {"int",              "int",           "I", "S"},
   {"Int",              "int",           "I", "S"},
   {"int*",             "Short",         "Ljava/lang/Short;"},
   {"Int*",             "Short",         "Ljava/lang/Short;"},
   {"IntPtr",           "Short",         "Ljava/lang/Short;"},
   {"ListPtr",          "int",           "I"},
   {"LocalID",          "int",           "I"},
   {"LocalID*",         "Integer",       "Ljava/lang/Integer;"},
   {"Long",             "int",           "I"},
   {"MenuBarPtr",       "int",           "I"},
   {"PointType*",       "PointType",     "Lpalmos/PointType;"},
   {"Ptr",              "Object",        "Ljava/lang/Object;"},
   {"RectanglePtr",     "Rectangle",     "Lpalmos/Rectangle;"},
   {"ScrOperation",     "int",           "I", "B"},
   {"SerSettingsPtr",   "SerSettings",   "Lpalmos/SerSettings;"},
   {"short",            "int",           "I", "S"},
   {"shortPtr",         "Short",         "Ljava/lang/Short;"},
   {"SlkPktHeaderPtr",  "SlkPktHeader",  "Lpalmos/SlkPktHeader;"},
   {"SlkSocketListenPtr", "SlkSocketListenPtr", "Lpalmos/SlkSocketListenPtr;"},
   {"SlkWriteDataPtr",  "SlkWriteData",  "Lpalmos/SlkWriteData;"},
   {"SndSysBeepType",   "int",           "I", "B"},
   {"SndCommandPtr",    "SndCommand",    "Lpalmos/SndCommand;"},
   {"SWord",            "int",           "I", "S"},
   {"SWord*",           "Short",         "Ljava/lang/Short;"},
   {"SWordPtr",         "Short",         "Ljava/lang/Short;"},
   {"SysBatteryKind*",  "Short",         "Ljava/lang/Short;"},
   {"SystemPreferencesPtr", "SystemPreferences", "Lpalmos/SystemPreferences;"},
   {"TableItemStyleType", "int",         "I", "B"},
   {"TablePtr",         "int",           "I"},
   {"TimeFormatType",   "int",           "I", "B"},
   {"UBytePtr",         "Byte",          "Ljava/lang/Byte;"},
   {"UInt",             "int",           "I", "S"},
   {"ULong",            "int",           "I"},
   {"ULongPtr",         "Integer",       "Ljava/lang/Integer;"},
   {"UnderlineModeType", "int",          "I"},
   {"void*",            "Object",        "Ljava/lang/Object;"},
   {"VoidHand",         "int",           "I"},
   {"voidptr",          "int",           "I"},
   {"VoidHand*",        "Integer",       "Ljava/lang/Integer;"},
   {"VoidPtr",          "Object",        "Ljava/lang/Object;"},
   {"UIntPtr",          "Short",         "Ljava/lang/Short;"},
   {"WindowFormatType", "int",           "I", "B"},
   {"WinHandle",        "int",           "I"},
   {"Word",             "int",           "I", "S"},
   {"WordPtr",          "Short",         "Ljava/lang/Short;"}
  };


  public static void main(String[] args) throws IOException
  {
    BufferedReader in = new BufferedReader (new FileReader 
					    (new File ("tools", "palmos.api")));
    PrintWriter jout = new PrintWriter (new BufferedWriter
					(new FileWriter 
					 (new File ("palmos", "Palm.java"))));
    PrintWriter aout = new PrintWriter (new BufferedWriter
					(new FileWriter ("native-palmos.asm")));

    jout.println("package palmos;");
    jout.println();
    jout.println("public class Palm {");
    while (true) {
      String s = in.readLine();
      if (s == null) {
        break;
      }
      if (s.length() == 0 || s.charAt(0) == '/') {
        continue;
      }
      if (s.startsWith("public ")) {
        while (true) {
          jout.println(s);
          if (s.charAt(0) == '}') {
            break;
          }
          s = in.readLine();
        }
        continue;
      }
      StringTokenizer st = new StringTokenizer (s, "(), ");

      // ======== analyze input line (one API function) ========

      // --- return type ---

      String rettype = st.nextToken();
      int retnum = -1;

      for (int i=0; i<RET_XLATE.length; i++) {
	if (RET_XLATE[i][0].equals (rettype)) {
	  retnum = i;
	  break;
	}
      }
      
      if (retnum < 0) {
	System.out.println("Unknown return type (table): " + rettype);
	System.exit(1);
      }

      // --- method name ---

      String methodName = st.nextToken();
      boolean lib = false;
      if (methodName.charAt(0) == '&') {
        methodName = methodName.substring(1);
        lib = true;
      }

      // --- parameters ---

      Vector params = new Vector();
      while (true) {
        ParamInfo param = new ParamInfo();
	String type = st.nextToken();

	if (type.charAt(0) == ';') {
          break;
        }
        
	param.type = type;
	param.num = -1;
	for (int i=0; i<PARAM_XLATE.length; i++) {
	  if (PARAM_XLATE[i][0].equals (type)) {
	    param.num = i;
	    break;
	  }
	}

	if (param.num < 0) {
	  System.out.println("Unknown parameter type (table): " + type);
          System.exit(1);
	}

        param.name = st.nextToken();
	params.addElement (param);
      }

      // ======== write Java ========

      jout.print("  public native static " + 
		 RET_XLATE[retnum][1] + " " + 
		 methodName + "(");

      for (int i=0; i<params.size(); i++) {
	ParamInfo param = (ParamInfo) params.elementAt(i);
        if (i>0) {
          jout.print(", ");
        }
        jout.print(PARAM_XLATE[param.num][1] +
		   " " + param.name);
      }
      jout.println(");");

      // ======== write assembly ========

      // --- write method-declaration comment line ---
      
      aout.print (";; native-method palmos/Palm." +
		  methodName + "(");
      for (int i=0; i<params.size(); i++) {
	ParamInfo param = (ParamInfo) params.elementAt(i);
	aout.print (PARAM_XLATE[param.num][2]);
      }
      aout.println (")" + RET_XLATE[retnum][2]);

      // --- write dependency comment lines ---
      
      for (int i=0; i<params.size(); i++) {
	String psig = PARAM_XLATE [((ParamInfo) params.elementAt(i)).num][2];
	if (psig.equals ("Ljava/lang/String;")) {
	}
	else if (psig.equals ("Ljava/lang/StringBuffer;")) {
	  aout.println (";; uses-field java/lang/StringBuffer.value as StringBuffer_value");
	}
	else if (psig.equals ("Ljava/lang/Object;")) {
	  
	}
	else if (psig.charAt(0) == 'L') {
	  aout.println (";; needs-exact-layout " + 
			psig.substring (1, psig.length()-1));
	}
      }

      // --- write instructions (parameter handling) ---
      
      aout.println("\tlink\ta6,#0");
      int psize = 0;
      for (int i=0; i<params.size(); i++) {
	ParamInfo param = (ParamInfo) params.elementAt(params.size()-i-1);
	String[] line = PARAM_XLATE[param.num];
	String sig = line[2];
	String pSig = (line.length >= 4 ? line[3] : sig);

	// argument number i is found at 'nnn(a6)'
	// with nnn = 8 + 4*i

	if (pSig.equals("Ljava/lang/String;")) {
          aout.println("\tmove.l\t" + (8+4*i) + "(a6),a0");
          aout.println("\tbsr\tStringToChars");
          aout.println("\tmove.l\ta0,-(a7)");
          psize += 4;
        }
	else if (pSig.equals("Ljava/lang/StringBuffer;")) {
          aout.println("\tmove.l\t" + (8+4*i) + "(a6),a0");
          aout.println("\tmove.l\tStringBuffer_value(a0),a0");
          aout.println("\tmove.l\tArray.data(a0),-(a7)");
          psize += 4;
        }
	else if (pSig.equals("Ljava/lang/Object;")) {
          aout.println("\tmove.l\t"+(8+4*i)+"(a6),-(a7)");
          aout.println("\tbsr\tmakevoidptr");
          psize += 4;
        }
	else if (pSig.startsWith("[")) {
          aout.println("\tmove.l\t"+(8+4*i)+"(a6),a0");
          aout.println("\tmove.l\tArray.data(a0),-(a7)");
          psize += 4;
        }
	else if (pSig.equals("I") ||
		   pSig.equals("F") ||
		   pSig.startsWith("L")) {
	  aout.println("\tmove.l\t"+(8+4*i)+"(a6),-(a7)");
          psize += 4;
        }
	else if (pSig.equals("S")) {
	  aout.println("\tmove.w\t"+(8+4*i+2)+"(a6),-(a7)");
          psize += 2;
        }
	else if (pSig.equals("B") ||
		   pSig.equals("C") ||
		   pSig.equals("Z")) {
	  aout.println("\tmove.b\t"+(8+4*i+3)+"(a6),-(a7)");
          psize += 2;
        }
	else {
          System.out.println("Unknown parameter type (asm): " + param.type);
          System.exit(1);
        }
      }

      // --- write instructions (systrap call) ---
      aout.println("\ttrap\t#15");
      if (lib) {
        aout.println("\tdc.w\tsysLibTrap" + methodName);
      } else {
        aout.println("\tdc.w\tsysTrap" + methodName);
      }
      if (psize > 0) {
        if (psize > 8) {
          aout.println("\tlea\t"+psize+"(a7),a7");
        } else {
          aout.println("\taddq.l\t#"+psize+",a7");
        }
      }

      // --- CharBuf / StringBuffer post-processing ---
      for (int i=0; i<params.size(); i++) {
	ParamInfo param = (ParamInfo) params.elementAt(params.size()-i-1);
        if (param.type.equals("CharBuf")) {
          aout.println("\tmove.l\td0,d3");
          aout.println("\tmove.l\t"+(8+4*i)+"(a6),a2");
          aout.println("\tmove.l\t(a2),a0");
          aout.println("\tsystrap\tStrLen(Array.data(a0).l)");
          aout.println("\tmove.w\td0,4+2(a2)");
          aout.println("\tmove.l\td3,d0");
        }
      }

      // --- write instructions (return-value handling) ---
      String retSig = RET_XLATE[retnum][2];
      String pRetSig = (RET_XLATE[retnum].length >= 4 ? 
			RET_XLATE[retnum][3] : retSig);

      if (pRetSig.equals("Z")) {
        aout.println("\ttst.b\td0");
        aout.println("\tsne.b\td0");
        aout.println("\tand.l\t#1,d0");
      } 
      else if (pRetSig.equals("C")) {
        aout.println("\tand.l\t#$ff,d0");
      } 
      else if (pRetSig.equals("B")) {
        aout.println("\text.w\td0");
	aout.println("\text.l\td0");
      } 
      else if (pRetSig.equals("B+")) {
        aout.println("\tand.l\t#$ff,d0");
      } 
      else if (pRetSig.equals("S")) {
        aout.println("\text.l\td0");
      } 
      else if (pRetSig.equals("S+")) {
        aout.println("\tand.l\t#$ffff,d0");
      } 
      else if (pRetSig.equals("I")) {
        // nothing
      } 
      else if (pRetSig.equals("I+")) {
        // nothing
      } 
      else if (pRetSig.equals("L")) {
	if (!retSig.startsWith("L")) {
	  aout.println("\tmove.l\ta0,d0");
	} else {
	  // we should wrap the result into a Jump Object
	  ASSERT.fail ("wrapping of a PalmOS " + pRetSig + 
		       " into a Java " + retSig +
		       " not implemented.");
	}
      } 
      else if (pRetSig.equals("Ljava/lang/String;")) {
        aout.println("\tbsr\tCharPtr_to_String");
      } 
      else if (pRetSig.equals("V")) {
        // nothing
      } 
      else {
        System.out.println("Unknown return type (asm): " + rettype + 
			   " / " + retSig + " / " + pRetSig);
        System.exit(1);
      }
      aout.println("\tunlk\ta6");
      aout.println("\trts");
      aout.println();
      System.out.println("  "+methodName);
    }
    jout.println("}");
    jout.close();
    aout.close();
  }
}
