/*
The MenuProgram.java is written by Noli Sicad and Ralf Kleberhoff
demonstrate the power of JUMP2 in implementing Palm OS Programs 
in Java without the virtual machine e.g. WabaVM, KVM etc.

Noveber 11, 2000
*/


import palmos.*;

public class MenuProgram {
  static final int FORM1_ID    = 1004;
  static final int FORM2_ID    = 1026;
  
  static final int BUTTON1_ID  = 1028;
  static final int BUTTON2_ID  = 1027;
  
  static final int MENU1     = 1042;
  static final int ItemMenuForm2 = 1061;
  static final int ItemAlertAbout = 1064;
  
  static final int ALERTID_ABOUT = 1063;

  public static int PilotMain(int cmd, int cmdBPB, int launchFlags)
  {
    if (cmd == 0) {
      new MenuProgram().run();
    }
    return 0;
  }

  public void run()
  {
    Palm.FrmGotoForm(FORM1_ID);
    Event e = new Event();
    Short err = new Short((short)0);
    while (e.eType != Event.appStopEvent) {
      Palm.EvtGetEvent(e, 0);
      if (!Palm.SysHandleEvent(e)) {
        if (!Palm.MenuHandleEvent(0, e, err)) {
          if (!appHandleEvent(e)) {
            Palm.FrmHandleEvent(Palm.FrmGetActiveForm(), e);
          }
        }
      }
    }
  }

  boolean appHandleEvent(Event e)
  {
    if (form1HandleEvent(e)) {
      return true;
    }
    else if (form2HandleEvent(e)) {
      return true;
    }
    return false;
  }

  boolean form1HandleEvent(Event e)
  {
    if (e.eType == Event.frmLoadEvent) {
      if (e.formID() != FORM1_ID) {
	return false;
      }
      int form = Palm.FrmInitForm(FORM1_ID);
      Palm.FrmSetActiveForm(form);
      return true;
    }
    else if (e.eType == Event.frmOpenEvent) {
      if (e.formID() != FORM1_ID) {
	return false;
      }
      Palm.FrmDrawForm(Palm.FrmGetFormPtr(FORM1_ID));
      return true;
    }
	else if (e.eType == Event.menuEvent) { 
	  menuMainEvent(e);
      return true;
       }	
    else if (e.eType == Event.ctlSelectEvent) {
      if (e.controlID() ==BUTTON1_ID) {
	Palm.FrmGotoForm(FORM2_ID);
        return true;
      }
    }
    return false;
  }


  boolean menuMainEvent(Event e )
  {
   if (e.eType == Event.menuEvent) {
      if (e.itemID() == ItemMenuForm2) {
        int form = Palm.FrmInitForm(FORM2_ID);
        Palm.FrmDoDialog(form);
		Palm.FrmDeleteForm(form);
		return true;
      }
      else if (e.itemID() == ItemAlertAbout) {
	    Palm.FrmAlert(ALERTID_ABOUT);
        return true;
      }
   }  
   return false;
  }
  
  boolean form2HandleEvent(Event e)
  {
    if (e.eType == Event.frmLoadEvent) {
      if (e.formID() != FORM2_ID) {
	return false;
      }
      int form = Palm.FrmInitForm(FORM2_ID);
      Palm.FrmSetActiveForm(form);
      return true;
    }
    else if (e.eType == Event.frmOpenEvent) {
      if (e.formID() != FORM2_ID) {
	return false;
      }
      Palm.FrmDrawForm(Palm.FrmGetFormPtr(FORM2_ID));
      return true;
    }
    else if (e.eType == Event.ctlSelectEvent) {
      if (e.controlID() ==BUTTON2_ID) {
	Palm.FrmGotoForm(FORM1_ID);
        return true;
      }
    }
    return false;
  }

}
